/* ---------------------------------------------------------------------------
 * Console for easily doing easy TIM things.
 * Adapted (copied!) from 'timlet' and 'timrun' programs written by J.B.Lane.
 *
 * Matthew Warren Oct 2004                                        timConsole.c
 * ---------------------------------------------------------------------------
 */


/* ox9 PLD firmware is CVS v1.30 14Oct02 */


#include <time.h>

#include "ask.h"
#include "vme.h"
#include "timlet.h"
#include "signal.h"


#define TIM_CMD_SIZE  100 /* 37 too small, 50 too small by 1 */
#define REGISTER_WORDS 26

static int reset_level = 0 ;

static int exit_loop = 0;

const static unsigned short RAM_mask[1] = {0xFFFF} ;

unsigned int buffer[TIM_RAM_SIZE] ;

void timConsole    (void) ;
void timShowStatus (void);
void test_d (void);
void test_e (void);
void show_FFTV_status (void);


int main (void)
{
  printf("\n=== timConsole ===== (not rocket-science) ======\n\n");
	
  (void) tim_vme (0x10000) ;
  ram_ptr    = memory_ptr + TIM_RAM_SIZE ; /* offset is 16-bit words */

  if (memory_ptr != NULL) timConsole () ;
  else printf ("Error opening VME window\n") ;

  vme_end () ;
  return (0) ;
}

/* ---------------------------------------------------------------------------
 */
void signal_handler (int sig)
{
	exit_loop = 1;
   printf ("\nExiting loop ASAP\n") ;
}

/* ---------------------------------------------------------------------------
 */
void timConsole (void)
{
	int ncommand = 0;
	int quit = 0;
	int indexL1A; // for check_Sink
	int i;
	unsigned long u1, u2 ;
		
	timSetupPtr();
	profile_tim();
	
	printf("================================================\n");
	
	while (quit < 1)
	{
		
		printf("\n-- Status --------------------------------------\n");
	 	
	 	timShowStatus();

		printf("\n-- Menu ----------------------------------------\n");
		printf("  0. Check Status\n");
		printf("  1. Reset TIM\n");	
		printf("  2. Enable TTC Clock (fibre)\n");	
		printf("  3. Enable External Clock (front panel lemo)\n");	
		printf("  4. Enable Internal Clock (disable Ext/TTC Clocks)\n");
		printf("  5. Set RunMode\n");		
		printf("  6. Run All signals from Sequencer (cyclic)\n");
		printf("  7. Reset Sink - Ready for Triggers\n");
		printf("  8. Dump Sink RAM\n");
		printf("  9. Quit\n");
		printf("\n");
		printf("  a. TTC to SA Repeditive Clock Switch Test (for scoping)\n");
		printf("  b. Run FFTV Random/Fixed Freq Test\n");
		printf("  c. Run FFTV Enable/Disable Test\n");

		printf("  d. Run Random Trigs, FFT Count Monitor Test\n");
		printf("  e. Run FFTV scan test\n");
		printf("  f. \n");
						
		printf("\n");

		(void) ask_int("\nEnter Choice (<Enter> defaults to check status): ", 0, &ncommand);
		printf("\n");
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 1)
		{
			timReset(Loud);
		}
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 2)
		{
			printf("Enabling TTC Clock (+ disabling ExtClk).\n");
			*reg_enables = (int)*reg_enables & 0xFEFF;
			*reg_EnRun = (int)*reg_EnRun | 0x0001;
		}
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 3)
		{
			printf("Enabling External Clock (+ disabling TTCClk).\n");
			*reg_EnRun = (int)*reg_EnRun & 0xFFFE;
			*reg_enables = (int)*reg_enables | 0x0100;
		}
		
		
		/* ------------------------------------------------------------------- */	
		if (ncommand == 4)
		{
			printf("Enabling Internal Clock (by disabling TTC/Ext Clocks).\n");
			*reg_EnRun = (int)*reg_EnRun & 0xFFFE;
			*reg_enables = (int)*reg_enables & 0xFEFF;
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 5)
		{
			printf("Setting RunMode.\n");
			*reg_command = (int)*reg_command | 0x1000; // setrunmode
		}


		/* ------------------------------------------------------------------- */
		if (ncommand == 6)
		{
			printf("Leaving all signals running from Seq in cyclic mode.\n");
			
			for (i = 0 ; i < 10 ; i++)
				ram_ptr[i] = 0x0000 ;

			ram_ptr[1] = 0x0001;
			ram_ptr[2] = 0x0002;
			ram_ptr[3] = 0x0004;
			ram_ptr[4] = 0x0008;
			ram_ptr[5] = 0x0010;
			ram_ptr[6] = 0x0020;
			ram_ptr[7] = 0x0040;
			ram_ptr[8] = 0x0080;

			*reg_seq_end = 0x0009;
			*reg_sequenc = 0x2200; //reset
			*reg_sequenc = 0x0CFF; //en-cyclic, go, all-sigs!
		}
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 7)
		{
			printf("Init Sequencer, ready for Trigger.\n");
			
			*reg_seq_end = 0x3FFF; //end value
			*reg_sequenc = 0x2200; //reset
			*reg_sequenc = 0x8000; //wait for next trig
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 8)
		{
			printf("Scanning the SinkRAM (checkSink).\n");
			checkSink (TIM_RAM_SIZE, NONE, &indexL1A);
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 9)
		{
			printf("Byeeeeee!\n\n");
			quit = 1;
		}
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 0xa)
		{
			printf("Switching between TTC and Stand-alone clock (currently ");

			if (reg[ENABLES] & bEnExtCLK) printf("External"); 
			else								  printf("Internal");
			
 			printf(") via PLL-stable bit\n");
			printf("Exit loop with Control-C\n") ;

			/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	   	if (signal (SIGINT, signal_handler) == SIG_ERR)
         	printf ("OS signal error (trying to catch ^C)\n") ;
 	      	      
        	exit_loop = 0;     
         while (exit_loop == 0) {
	         
	         timSwitchClk(TTC);
				timSwitchClk(Standalone);
          }

			/* Treat the next Control-C in default way (not needed on NT) */
          if (signal (SIGINT, SIG_DFL) == SIG_ERR)
              printf ("Os signal error (trying to un-catch ^C)\n") ;

		}
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 0xb)
		{
			printf("Testing for FFTV by switch from Random to Fixed-Freq Trigs.\n");

			printf("Exit loop with Control-C\n") ;
			
			reg[FV_MATCH]  = 10;
			reg[FV_P_MIN]  = convHz2Clk(600000);
			reg[FV_P_MAX]  = convHz2Clk(110000);
			reg[FV_OFLOW]  = convHz2Clk(100000);
			reg[FV_DELTAT] = 2;
			
			printf("\n\n");
			printf("FFTV Settings:\n");
			printf(" N Matches (FV_MATCH)         %d\n", reg[FV_MATCH]);
			printf(" Upper Freq Limit (FV_P_MIN)  %d (%d kHz)\n", reg[FV_P_MIN], convClk2Hz(reg[FV_P_MIN])/1000);
			printf(" Lower Freq Limit (FV_P_MAX)  %d (%d kHz)\n", reg[FV_P_MAX], convClk2Hz(reg[FV_P_MAX])/1000);
			printf(" Tolerance (FV_DELTAT)        %d (%d ms)\n", reg[FV_DELTAT], convClk2Sec(reg[FV_DELTAT])*1000);
			printf(" Veto Duration (FV_VETOLEN)   %d (%d ms)\n", reg[FV_VETOLEN], convClk2Sec(reg[FV_VETOLEN])*1000);
			
			u1 = /* reg[FV_COUNTX]*0x100000000 + */
					reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL];
					
			u2 = /*reg[BCOUNTX]*0x100000000 + */
			
					reg[BCOUNTH]*0x10000 + reg[BCOUNTL];

			printf(" FFTV Count (clks)(FV_COUNT*) %I64u\n", u1);
			printf(" Busy Count (clks)(BCOUNT*)   %I64u\n", u2);
				
				
			reg[DEBUG_CTL] |= bFvDisable;	 /* FFTV Off */				
			reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	
			

			
			/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	   	if (signal (SIGINT, signal_handler) == SIG_ERR) {
         	printf ("OS signal error (trying to catch ^C)\n") ;
         }
 	      	      
        	exit_loop = 0;     
         while (exit_loop == 0) {
	         

				reg[ENABLES] &= ~bEnRandom;	/* Randomiser off */
				reg[FREQUENCY] = 0x04; 			/* 150kHz */

				(void) wait_state_timeout(50, 0, 0, 1); /* just a wait 0,0 will never be 1) */


				reg[ENABLES] |= bEnRandom; 	/* Randomiser on */
				reg[FREQUENCY] = 0x00; 			/* 600kHz - Randomiser reduces average rate by factor of 4 */
												
				(void) wait_state_timeout(50, 0, 0, 1);

         }

			/* Treat the next Control-C in default way (not needed on NT) */
         if (signal (SIGINT, SIG_DFL) == SIG_ERR) {
         	printf ("Os signal error (trying to un-catch ^C)\n") ;
         }
              
			printf("\n\n");
			printf("FFTV Settings:\n");
			printf(" N Matches (FV_MATCH)         %d\n", reg[FV_MATCH]);
			printf(" Upper Freq Limit (FV_P_MIN)  %d (%d kHz)\n", reg[FV_P_MIN], convClk2Hz(reg[FV_P_MIN])/1000);
			printf(" Lower Freq Limit (FV_P_MAX)  %d (%d kHz)\n", reg[FV_P_MAX], convClk2Hz(reg[FV_P_MAX])/1000);
			printf(" Tolerance (FV_DELTAT)        %d (%d ms)\n", reg[FV_DELTAT], convClk2Sec(reg[FV_DELTAT])*1000);
			printf(" Veto Duration (FV_VETOLEN)   %d (%d ms)\n", reg[FV_VETOLEN], convClk2Sec(reg[FV_VETOLEN])*1000);
			
			
			u1 = /* reg[FV_COUNTX]*0x100000000 +*/ 
					reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL];
			printf(" FFTV Count (clks)(FV_COUNT*) %lu\n", u1);
			
			u1 = reg[BCOUNTX]*0xFFFFFFFF + reg[BCOUNTH]*0xFFFF + reg[BCOUNTL];
			printf(" Busy Count (clks)(BCOUNT*)   %lu\n", u1);
	                       
	
		}
	
		/* ------------------------------------------------------------------- */
		if (ncommand == 0xc)
		{
			printf("Testing for FFTV by switch FFTV on and off.\n");

			printf("Exit loop with Control-C\n") ;
			
			reg[FV_MATCH]   = 10;
			reg[FV_P_MIN]   = convHz2Clk(600000);
			reg[FV_P_MAX]   = convHz2Clk(110000);
			reg[FV_OFLOW]   = convHz2Clk(100000);
			reg[FV_DELTAT]  = 2;
			reg[FV_VETOLEN] = 0x7FFF;
			
			
			printf("\n\n");
			printf("FFTV Settings:\n");
			printf(" N Matches (FV_MATCH)         %d\n", reg[FV_MATCH]);
			printf(" Upper Freq Limit (FV_P_MIN)  %d (%d kHz)\n", reg[FV_P_MIN], convClk2Hz(reg[FV_P_MIN])/1000);
			printf(" Lower Freq Limit (FV_P_MAX)  %d (%d kHz)\n", reg[FV_P_MAX], convClk2Hz(reg[FV_P_MAX])/1000);
			printf(" Tolerance (FV_DELTAT)        %d (%d ms)\n", reg[FV_DELTAT], convClk2Sec(reg[FV_DELTAT])*1000);
			printf(" Veto Duration (FV_VETOLEN)   %d (%d ms)\n", reg[FV_VETOLEN], convClk2Sec(reg[FV_VETOLEN])*1000);
						
			reg[ENABLES] |= bEnExtTRIG; 	/* Ext Trigs on */	

			
			/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	   	if (signal (SIGINT, signal_handler) == SIG_ERR)
         	printf ("OS signal error (trying to catch ^C)\n") ;
 	      	      
        	exit_loop = 0;     
         while (exit_loop == 0) {
	         

				reg[DEBUG_CTL] |= bFvDisable;	 /* FFTV Off */

				(void) wait_state_timeout(50, 0, 0, 1); /* just a wait 0,0 will never be 1) */


				reg[DEBUG_CTL] &= ~bFvDisable;	/* FFTV On */

												
				(void) wait_state_timeout(50, 0, 0, 1);

         }

			/* Treat the next Control-C in default way (not needed on NT) */
         if (signal (SIGINT, SIG_DFL) == SIG_ERR)
         	printf ("Os signal error (trying to un-catch ^C)\n") ;
              
                      

		}
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 0xd)
		{
			test_d();
		}

		/* ------------------------------------------------------------------- */
		if (ncommand == 0xe)
		{
			test_e();
		}
	}
}




/* ---------------------------------------------------------------------------
 */
void test_d(void) {
	
	int quit = 0;
	int i2;
	unsigned long u1, u2 ;
  	time_t time_start, time_now;
  	double dift;

	printf("Monitoring for Veto'd Trigs when running Randoms.\n");
	printf("Exit loop with Control-C\n") ;
		
	reg[FV_TCOUNT] = 0;
	reg[FV_COUNTL] = 0;

	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(40000000);
	reg[FV_P_MAX]   = convHz2Clk(15000);
	reg[FV_OFLOW]   = convHz2Clk(15000);
	reg[FV_DELTAT]  = 20;
	reg[FV_VETOLEN] = 40000; /* 1ms */

	
	show_FFTV_status();

	/* ---------------------------------------------------- */

	printf("\nRunning a few non-random trigs to prove working (osc=60kHz)\n");
	
	reg[DEBUG_CTL] = 0x1000; /* ensure FFTV is reset */
	reg[DEBUG_CTL] = 0x0000; /* and on */


	reg[ENABLES] &= ~bEnRandom; 	/* Randomiser off */
	reg[FREQUENCY] = 0x01; 			/* 60kHz */

	time ( &time_start );

	reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	

	for(u1=0; u1<100000000; u1++) {} /* delay of 1s on PC25 running NT! */

	u1 = reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*+ reg[FV_COUNTX]*0x100000000*/
	
	time ( &time_now );
	dift = difftime(time_now, time_start);
	
	u2 = ((reg[L1IDH]&0x00FF)*0x10000) + reg[L1IDL];
	printf("Secs %04x, FVTCount %04x, L1ID %7lu, FVCount %7lu\n", dift, reg[FV_TCOUNT], u2, u1);

	/* ---------------------------------------------------- */

	printf("\nSwitching to random trigs, reg's reset (osc=300kHz)\n\n");
	
	reg[ENABLES] &= ~bEnIntTRIG; 	/* Int Trigs off */				
	
	reg[FV_TCOUNT] = 0;
	reg[FV_COUNTL] = 0;
	reg[L1IDL] = 0;
	
	reg[FREQUENCY] = 0x02; 			/* 75kHz*4 = 300kHz */
	reg[ENABLES] |= bEnRandom; 	/* Randomiser on */


	reg[DEBUG_CTL] = 0x1000; /* ensure FFTV is reset */
	reg[DEBUG_CTL] = 0x0000; /* and on */

	time ( &time_start );
  	printf ( "Start timestamp: %s", ctime (&time_start) );

	reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	

	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
   	printf ("OS signal error (trying to catch ^C)\n") ;
   
  	exit_loop = 0;     
   while (exit_loop == 0) {
  
 
		u1 = reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*reg[FV_COUNTX]*0x100000000 + */
				
		if (u1 != u2) {
			time(&time_now);
			dift = difftime(time_now, time_start);
			u2 = ((reg[L1IDH]&0x00FF)*0x10000) + reg[L1IDL];
			i2 = reg[FV_TCOUNT];
			printf("Secs %d, FVTCount %d, L1ID %7lu, FVCount %7lu\n", dift, i2, u2, u1);

		}
		u2 = u1;

   }

	/* Treat the next Control-C in default way (not needed on NT) */
   if (signal (SIGINT, SIG_DFL) == SIG_ERR)
   	printf ("Os signal error (trying to un-catch ^C)\n") ;
   	
	time ( &time_now );
  	printf ( "End timestamp: %s", ctime (&time_now) );
}


/* ---------------------------------------------------------------------------
 */

void test_e(void)
{
	int l1id, oldl1id, i;
	unsigned long  fvcount, oldfvcount ;
  	time_t time_now;

	printf("Monitoring for Veto'd Trigs when running Randoms.\n");
	printf("Exit loop with Control-C\n") ;
			
	reg[FV_TCOUNT] = 0;
	reg[FV_COUNTL] = 0;

	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(40000000);
	reg[FV_P_MAX]   = convHz2Clk(15000);
	reg[FV_OFLOW]   = convHz2Clk(15000);
	reg[FV_DELTAT]  = 40;
	reg[FV_VETOLEN] = 40000; /* 1ms */
	
	show_FFTV_status();	

	/* ---------------------------------------------------- */
	
	reg[FREQUENCY] = 0x02; 			/* 75kHz*4 = 300kHz */
	reg[ENABLES] |= bEnRandom; 	/* Randomiser on */

	time ( &time_now );
  	printf ( "Start timestamp: %s", ctime (&time_now) );
	
	for (i=6; i<11; i++) {
  	
  		reg[FV_MATCH] = i;
  		printf("\n\n--------------------------------------------------------------\n");
  		printf("New loop, FV_MATCH: %d.\n\n",reg[FV_MATCH]);
  		
		reg[DEBUG_CTL] = 0x1000; /* ensure FFTV is reset */
		reg[DEBUG_CTL] = 0x0000; /* and on */

  		reg[L1IDL] = 0; /* actually think they reset to 0xfffff by write */
  		reg[L1IDH] = 0;

  		reg[FV_TCOUNT] = 0;
		reg[FV_COUNTL] = 0;
	  		
		l1id = 0;
		oldl1id = 0;
		
		fvcount = 0;
		oldfvcount = 0;

		time ( &time_now );
  		printf ( "FVMatch %d, Timestamp: %s", i, ctime (&time_now) );
  		
		reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	
	   
	   while (l1id < 16000000) { /* >= oldl1id) { */

	 		fvcount = (int)reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*reg[FV_COUNTX]*0x100000000 + */
	 		l1id    = timGetL1ID(reg[L1IDL], reg[L1IDH]);
					
			if (fvcount > (oldfvcount+(11-i)*10000000) ) {
				time(&time_now);
				
				printf("FVMatch %d,  L1ID %7lu,  FVCount %7lu\n", i, l1id, fvcount);
				oldfvcount = fvcount;
			}

			oldl1id = l1id;
	   }
	   
	   reg[ENABLES] &= ~bEnIntTRIG; 	/* Int Trigs off */
	   
 		fvcount = (int)reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*reg[FV_COUNTX]*0x100000000 + */
 		l1id    = timGetL1ID(reg[L1IDL], reg[L1IDH]);
		printf("FVMatch %d,  L1ID %7lu,  FVCount %7lu\n", i, l1id, fvcount);

	   
	   time ( &time_now );
  		printf ( "FVMatch %d, Timestamp: %s", i, ctime (&time_now) );

	}
}


/* ---------------------------------------------------------------------------
 */

void timShowStatus(void)
{
	
	//printf("Status Not Working\n");
	
	int l1id;
	int status;

	(void)check_clk();
	(void)check_TTCclk();

	l1id = timGetL1ID( *reg_L1lo, *reg_L1hi );
	status =(int)*reg_status;
	 /*
	//printf("TIM_OK: ");
	//if ((status & TIM_OK) == 0) printf("NOT OK!"); else printf("OK!");

	printf("Internal Clock: ");
	if ((status & SAclkON) == 0) printf("FAILED!") else printf("Detected");

	printf("/n");
	printf("External Clock: ");
	if ((enables & 0x0100) == 0) printf("Disabled") else printf("Enabled");



 //Stand-alone clock OFF, status %04X, delays %04X\n",
   //         status, *reg_delays) ;
    ///error = 1 ;
  //}
//printf("TTC Clock :");
	

*/
	
	printf("Status Reg:%04x,  RunEnables:%04x,  Output:%04x.\n",
			*reg_status,*reg_EnRun,*reg_output);   
	printf("L1ID:%06x,  BCID:%03x,  BCin:%03x,  TTID:%03x.\n", 
	 			l1id, *reg_BCid, *reg_BCin, *reg_TTid);

}


void show_FFTV_status() {

	printf("\n\n");
	printf("FFTV Settings:\n");
	printf(" N Matches (FV_MATCH)         %d\n", reg[FV_MATCH]);
	printf(" Upper Freq Limit (FV_P_MIN)  %d (%d kHz)\n", reg[FV_P_MIN], convClk2Hz(reg[FV_P_MIN])/1000);
	printf(" Lower Freq Limit (FV_P_MAX)  %d (%d kHz)\n", reg[FV_P_MAX], convClk2Hz(reg[FV_P_MAX])/1000);
	printf(" Tolerance (FV_DELTAT)        %d (%d ms)\n", reg[FV_DELTAT], convClk2Sec(reg[FV_DELTAT])*1000);
	printf(" Veto Duration (FV_VETOLEN)   %d (%d ms)\n", reg[FV_VETOLEN], convClk2Sec(reg[FV_VETOLEN])*1000);

}
