/* Note: TIM registers are in format: reg[REG_NAME]; */


/* TTCvi register pointers */
ttc_CSR1  = ttc_ptr + 0x80 / 2 ;
ttc_CSR2  = ttc_ptr + 0x82 / 2 ;
ttc_reset = ttc_ptr + 0x84 / 2 ;
ttc_trig  = ttc_ptr + 0x86 / 2 ;
ttc_L1hi  = ttc_ptr + 0x88 / 2 ;
ttc_L1lo  = ttc_ptr + 0x8A / 2 ;
ttc_0mod  = ttc_ptr + 0x90 / 2 ;
ttc_0del  = ttc_ptr + 0x92 / 2 ;
ttc_0dur  = ttc_ptr + 0x94 / 2 ;
ttc_Bgo0  = ttc_ptr + 0x96 / 2 ;
ttc_0cmd  = ttc_ptr + 0xB2 / 2 ;
ttc_ladd  = ttc_ptr + 0xC0 / 2 ;
ttc_long  = ttc_ptr + 0xC2 / 2 ;
ttc_acmd  = ttc_ptr + 0xC4 / 2 ;


/* TTCvi Setup */
/* ----------------------------------------------------------------*/


*ttc_reset = 0 ; 		 /* no data TTCvi reset */
*ttc_CSR1  = 0xC ; 	 /* VME L1A + Orbit */
*ttc_CSR2  = 0xFE00 ; /* reset + repeat 0 */


*ttc_0del  = 0 ; 		 /* Inhibit 0 delay */
*ttc_0dur  = 142 ; 	 /* Inhibit 0 duration */

*ttc_ladd  = 0x8000 ; /* long-format broadcast to TTCrx internal */
*ttc_long  = 0x03E3 ; /* send long-format data to sub-address 3 */


*ttc_0cmd  = 1 << 7 ; /* B-Go 0 data for BCR */
*ttc_0mod  = 0xD ;    /* B-Go 0 mode */




/* Test Loop */
/* ----------------------------------------------------------------*/


for (i = 0 ; i < nloops ; i ++ )  {
	
	data = loop & 0xFF ;
	
	*ttc_ladd  = 0x8000 ; /* long-format broadcast to TTCrx internal */
	*ttc_long  =   data ; /* send long-format data to sub-address 0 */

	reg[TTC_STATUS] = 0 ; /* clear 'catch/latching' register */
	reg[TIM_OUTPUT] = 0 ; /* clear 'catch/latching' register */


   type = reg[L1IDL] & 0xFF ;

	*ttc_ladd  = 0x8001 ; /* long-format broadcast to TTCrx external */
	*ttc_long  = type ; 	 /* send long-format data - 8-bit trigger type */
	*ttc_trig  = 0 ; 		 /* no data */


	l1id =  *ttc_L1lo + ((*ttc_L1hi & 0xFF) << 16) ;

	error += timCheck ("TTC : l1id   want", l1id, nextL1ID) ;
	nextL1ID = (l1id + 1) & 0xFFFFFF ;


	/* here we test to see if the L1A and DouStr have been seen by TIM */
	error += timCheck ("TTC : TIM_OUTPUT want", (reg[TIM_OUTPUT] & ~BCR), TRG) ; 					/* BCR maybe or not */
	error += timCheck ("TTC : TTC_STATUS(&F9FE) want", (reg[TTC_STATUS] & ~0x0601), 0x4900) ; /* Ready + DoutStr + Trig */
}
