/* ---------------------------------------------------------------------------
 * Console for easily doing easy TIM things.
 * Adapted (copied!) from 'timlet' and 'timrun' programs written by J.B.Lane.
 *
 * Matthew Warren Oct 2004                                        timConsole.c
 * ---------------------------------------------------------------------------
 */


/* ox9 PLD firmware is CVS v1.30 14Oct02 */


#include <time.h>

#include "ask.h"
#include "vme.h"
#include "timlet.h"
#include "signal.h"
#include <string.h> /* strcat strcpy */


static int reset_level = 0 ;
static int exit_loop = 0;

const static unsigned short RAM_mask[1] = {0xFFFF} ;

unsigned int buffer[TIM_RAM_SIZE] ;

void timConsole    (void) ;
void timShowStatus (void);
void test_1 (void);
void test_2 (void);
void test_3 (void);
void test_4 (void);
void test_5 (void);
void test_6 (void);
void test_7 (void);
void test_8 (void);
void test_9 (void);
void test_a (void);
void test_b (void);
void test_c (void);
void test_fftv (void);
void fftv_scan_test (void);
void test_f (void);
void send_ttcvi_random_trigs (void);
void ttc_ttype_test (void);
void test_tp_fifo (void);
void twin_trig_burst (void);
void send_slow_tim_trigs (void);

void show_FFTV_status (void);
void show_ttcvi_trigrate_status(int ttc_config_val, int *freq_hz);
void get_and_show_fv_info(	const int fv_match,
									int *l1id,
									int *fv_countx, int *fv_counth, int *fv_countl, 
									int *fv_tcount, unsigned int t, int *fvid
									);


int main (void)
{
	unsigned int vme_base_addr ;
  	int r ;
  	
	printf("\n=== timConsole ===== (not rocket-science) ======\n\n");
	
  	(void) tim_vme (0x10000) ;
  	

 	if (memory_ptr == NULL) {
 		printf ("Error opening TIM VME window. EXITING!\n") ;
	}
	else {
		
 		ram_ptr = memory_ptr + TIM_RAM_SIZE ; /* offset is 16-bit words */
 		
 		r = ask ("Enter TTCvi VME base address", VME_BASE_TTC, &vme_base_addr) ;
  		ttc_ptr = vme_get_window (0x0800, vme_base_addr, VME_MODE_A24) ;
  		
  		if (ttc_ptr == NULL) {
  			printf ("Error opening TTCvi VME window, EXITING!\n") ;
  		}
 		else {
 			timConsole () ;
 		}
 	}

  	vme_end () ;
  	return (0) ;
}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void signal_handler (int sig)
{
	exit_loop = 1;
   printf ("\nExiting loop ASAP\n") ;
}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void timConsole (void)
{
	int ncommand = 0;
	int quit = 0;
	int indexL1A; // for check_Sink
	
		
	timSetupPtr();

	ttcviSetupPtr();
	
	profile_tim();
	
	printf("================================================\n");
	
	while (quit < 1)
	{
		
		printf("\n-- Status --------------------------------------\n");
	 	
	 	timShowStatus();

		printf("\n-- Menu ----------------------------------------\n");
		printf("  0. Check Status\n");
		printf("  1. Reset TIM\n");	
		printf("  2. Enable TTC Clock (fibre)\n");	
		printf("  3. Enable External Clock (front panel lemo)\n");	
		printf("  4. Enable Internal Clock (disable Ext/TTC Clocks)\n");
		printf("  5. Set RunMode\n");		
		printf("  6. Run All signals from Sequencer (cyclic)\n");
		printf("  7. Reset Sink - Ready for Triggers\n");
		printf("  8. Dump Sink RAM\n");
		printf("  9. Quit\n");
		printf("\n");
		printf("  a. TTC to SA Repeditive Clock Switch Test (for scoping)\n");
		printf("  b. Run FFTV Random/Fixed Freq Test\n");
		printf("  c. Run FFTV Enable/Disable Test\n");

		printf("  d. Run Random Trigs, FFT Count Monitor Test\n");
		printf("  e. Run FFTV scan test\n");
		printf("  f. 10 Second IO Speed test\n");
		printf(" 10. Send TTCvi Random Trigs\n");
		printf(" 11. Test trigger Period FIFO\n");
		printf(" 12. Send Twin Trigs Burst\n");
		printf(" 13. TTC Type Test\n");
		printf(" 14. Send Slow TIM Trigs\n");

		printf("\n");

		(void) ask_int("\nEnter Choice (<Enter> defaults to check status): ", 0, &ncommand);
		printf("\n");
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 1) timReset(Loud);
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 2) {
			printf("Enabling TTC Clock (+ disabling ExtClk).\n");
			*reg_enables = (int)*reg_enables & 0xFEFF;
			*reg_EnRun = (int)*reg_EnRun | 0x0001;
		}
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 3) {
			printf("Enabling External Clock (+ disabling TTCClk).\n");
			*reg_EnRun = (int)*reg_EnRun & 0xFFFE;
			*reg_enables = (int)*reg_enables | 0x0100;
		}
		
		
		/* ------------------------------------------------------------------- */	
		if (ncommand == 4) {
			printf("Enabling Internal Clock (by disabling TTC/Ext Clocks).\n");
			*reg_EnRun = (int)*reg_EnRun & 0xFFFE;
			*reg_enables = (int)*reg_enables & 0xFEFF;
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 5) {
			printf("Setting RunMode.\n");
			*reg_command = (int)*reg_command | 0x1000; // setrunmode
		}

		/* ------------------------------------------------------------------- */
	
		if (ncommand == 6) test_6();		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 7) {
			printf("Init Sequencer, ready for Trigger.\n");
			
			*reg_seq_end = 0x3FFF; //end value
			*reg_sequenc = 0x2200; //reset
			*reg_sequenc = 0x8000; //wait for next trig
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 8) {
			printf("Scanning the SinkRAM (checkSink).\n");
			checkSink (TIM_RAM_SIZE, NONE, &indexL1A);
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 9) {
			printf("Byeeeeee!\n\n");
			quit = 1;
		}
		
/* ------------------------------------------------------------------- */
		if (ncommand == 0xa)	test_a();
		if (ncommand == 0xb) test_b();	
		if (ncommand == 0xc)	test_c();
		if (ncommand == 0xd)	test_fftv();
		if (ncommand == 0xe) fftv_scan_test();
		if (ncommand == 0xf) test_f();
		if (ncommand == 0x10) send_ttcvi_random_trigs();
		if (ncommand == 0x11) test_tp_fifo();
		if (ncommand == 0x12) twin_trig_burst();
		if (ncommand == 0x13) ttc_ttype_test();				
		if (ncommand == 0x14) send_slow_tim_trigs();
	
	}
}




/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_6(void) {
	
	int i;

	printf("Leaving all signals running from Seq in cyclic mode.\n");
			
	for (i = 0 ; i < 10 ; i++) ram_ptr[i] = 0x0000 ;

	ram_ptr[1] = 0x0001;
	ram_ptr[2] = 0x0002;
	ram_ptr[3] = 0x0004;
	ram_ptr[4] = 0x0008;
	ram_ptr[5] = 0x0010;
	ram_ptr[6] = 0x0020;
	ram_ptr[7] = 0x0040;
	ram_ptr[8] = 0x0080;

	*reg_seq_end = 0x0009;
	*reg_sequenc = 0x2200; //reset
	*reg_sequenc = 0x0CFF; //en-cyclic, go, all-sigs!
			
}


/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_a(void) {
	
	
	printf("Switching between TTC and Stand-alone clock (currently ");
	
	if (reg[ENABLES] & bEnExtClk) printf("External"); 
	else								  printf("Internal");
	
	printf(") via PLL-stable bit\n");
	printf("Exit loop with Control-C\n") ;
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;
		      
	exit_loop = 0;     
	while (exit_loop == 0) {
	   
	   timSwitchClk(TTC);
		timSwitchClk(Standalone);
	 }
	
	/* Treat the next Control-C in default way (not needed on NT) */
	 if (signal (SIGINT, SIG_DFL) == SIG_ERR)
	     printf ("Os signal error (trying to un-catch ^C)\n") ;
	

}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_b(void) {
	
	unsigned long u1, u2 ;

	
	printf("Testing for FFTV by switch from Random to Fixed-Freq Trigs.\n");
	
	printf("Exit loop with Control-C\n") ;
	
	reg[FV_MATCH]  = 10;
	reg[FV_P_MIN]  = convHz2Clk(600000);
	reg[FV_P_MAX]  = convHz2Clk(110000);
	reg[FV_OFLOW]  = convHz2Clk(100000);
	reg[FV_DELTAT] = 2;
	
	show_FFTV_status();	
	
	u1 = /* reg[FV_COUNTX]*0x100000000 + */
			reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL];
			
	u2 = /*reg[BCOUNTX]*0x100000000 + */
	
			reg[BCOUNTH]*0x10000 + reg[BCOUNTL];
	
	printf(" FFTV Count (clks)(FV_COUNT*) %I64u\n", u1);
	printf(" Busy Count (clks)(BCOUNT*)   %I64u\n", u2);
		
		
	reg[DEBUG_CTL] |= bFVdisable;	 /* FFTV Off */				
	reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	
	
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR) {
		printf ("OS signal error (trying to catch ^C)\n") ;
	}
		      
	exit_loop = 0;     
	while (exit_loop == 0) {
	   
	
		reg[ENABLES] &= ~bEnRandom;	/* Randomiser off */
		reg[FREQUENCY] = 0x04; 			/* 150kHz */
	
		(void) wait_state_timeout(50, 0, 0, 1); /* just a wait 0,0 will never be 1) */
	
	
		reg[ENABLES] |= bEnRandom; 	/* Randomiser on */
		reg[FREQUENCY] = 0x00; 			/* 600kHz - Randomiser reduces average rate by factor of 4 */
										
		(void) wait_state_timeout(50, 0, 0, 1);
	
	}
	
	/* Treat the next Control-C in default way (not needed on NT) */
	if (signal (SIGINT, SIG_DFL) == SIG_ERR) {
		printf ("Os signal error (trying to un-catch ^C)\n") ;
	}
	     
	show_FFTV_status();			
	
	u1 = /* reg[FV_COUNTX]*0x100000000 +*/ 
			reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL];
	printf(" FFTV Count (clks)(FV_COUNT*) %lu\n", u1);
	
	u1 = reg[BCOUNTX]*0xFFFFFFFF + reg[BCOUNTH]*0xFFFF + reg[BCOUNTL];
	printf(" Busy Count (clks)(BCOUNT*)   %lu\n", u1);

	
}



/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_c(void) {
	
	
	printf("Testing for FFTV by switch FFTV on and off.\n");
	
	printf("Exit loop with Control-C\n") ;
	
	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(600000);
	reg[FV_P_MAX]   = convHz2Clk(110000);
	reg[FV_OFLOW]   = convHz2Clk(100000);
	reg[FV_DELTAT]  = 2;
	reg[FV_VETOLEN] = 0x7FFF;
	
	show_FFTV_status();
				
	reg[ENABLES] |= bEnExtTRIG; 	/* Ext Trigs on */	
	
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;
		      
	exit_loop = 0;     
	while (exit_loop == 0) {
	   
	
		reg[DEBUG_CTL] |= bFVdisable;	 /* FFTV Off */
	
		(void) wait_state_timeout(50, 0, 0, 1); /* just a wait 0,0 will never be 1) */
	
	
		reg[DEBUG_CTL] &= ~bFVdisable;	/* FFTV On */
	
										
		(void) wait_state_timeout(50, 0, 0, 1);
	
	}
	
	/* Treat the next Control-C in default way (not needed on NT) */
	if (signal (SIGINT, SIG_DFL) == SIG_ERR)
		printf ("Os signal error (trying to un-catch ^C)\n") ;
	     
	             
	

}


/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_fftv(void) {
	
	int quit = 0;
	int i2;
	unsigned long u1, u2 ;
  	time_t time_start, time_now;
  	double dift;

	printf("Monitoring for Veto'd Trigs when running Randoms.\n");
	printf("Exit loop with Control-C\n") ;
		
	reg[FV_TCOUNTL] = 0;
	reg[FV_COUNTL] = 0;

	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(40000000);
	reg[FV_P_MAX]   = convHz2Clk(15000);
	reg[FV_OFLOW]   = convHz2Clk(15000);
	reg[FV_DELTAT]  = 20;
	reg[FV_VETOLEN] = 40000; /* 1ms */

	
	show_FFTV_status();

	/* ---------------------------------------------------- */

	printf("\nRunning a few non-random trigs to prove working (osc=60kHz)\n");
	
	reg[DEBUG_CTL] = 0x1000; 			/* ensure FFTV is reset */
	reg[DEBUG_CTL] = 0x0000; 			/* and on */


	reg[ENABLES] &= ~bEnRandom; 		/* Randomiser off */
	reg[FREQUENCY] = 0x01; 				/* 60kHz */

	time ( &time_start );

	reg[ENABLES] |= bEnIntTRIG; 		/* Int Trigs on */	

		(void) wait_state_timeout(1000, 0, 0, 1); /* (~1000ms) delay of 1s */

	u1 = reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*+ reg[FV_COUNTX]*0x100000000*/
	
	time ( &time_now );
	dift = difftime(time_now, time_start);
	
	u2 = ((reg[L1IDH]&0x00FF)*0x10000) + reg[L1IDL];
	printf("Secs %04x, FVTCountL %04x, L1ID %7lu, FVCount %7lu\n", dift, reg[FV_TCOUNTL], u2, u1);

	/* ---------------------------------------------------- */

	printf("\nSwitching to random trigs, reg's reset (osc=300kHz)\n\n");
	
	reg[ENABLES] &= ~bEnIntTRIG; 	/* Int Trigs off */				
	
	reg[FV_TCOUNTL] = 0;
	reg[FV_COUNTL] = 0;
	reg[L1IDL] = 0;
	
	reg[FREQUENCY] = 0x02; 			/* 75kHz*4 = 300kHz */
	reg[ENABLES] |= bEnRandom; 	/* Randomiser on */


	reg[DEBUG_CTL] = 0x1000; /* ensure FFTV is reset */
	reg[DEBUG_CTL] = 0x0000; /* and on */

	time ( &time_start );
  	printf ( "Start timestamp: %s", ctime (&time_start) );

	reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	

	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
   	printf ("OS signal error (trying to catch ^C)\n") ;
   
  	exit_loop = 0;     
   while (exit_loop == 0) {
  
 
		u1 = reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*reg[FV_COUNTX]*0x100000000 + */
				
		if (u1 != u2) {
			time(&time_now);
			dift = difftime(time_now, time_start);
			u2 = ((reg[L1IDH]&0x00FF)*0x10000) + reg[L1IDL];
			i2 = reg[FV_TCOUNTL];
			printf("Secs %d, FVTCountL %d, L1ID %7lu, FVCount %7lu\n", dift, i2, u2, u1);

		}
		u2 = u1;

   }

	/* Treat the next Control-C in default way (not needed on NT) */
   if (signal (SIGINT, SIG_DFL) == SIG_ERR)
   	printf ("Os signal error (trying to un-catch ^C)\n") ;
   	
	time ( &time_now );
  	printf ( "End timestamp: %s", ctime (&time_now) );
}


/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void fftv_scan_test(void)
{
	
	const int DEBUGLVL = 0;
	
	int i=0, mode=0, loop=0, max_loops=0;
	int l1id=0;
	int fv_counth=0,	 fv_countl=0,  fv_countx=0;
	int fvtcount=0;
	int fvid=0;
		 		 
	int freq_hz=0;
		
	double diff_time, old_diff_time;
	float rat;
	
	int 		al1id[15], 
				afv_counth[15],afv_countl[15], afv_countx[15], 
				afvtcount[15],
				afvid[15];
	double 	adiff_time[15];

	time_t time_now, start_time;
  	
  	int trig_en_bit=0, xtrig_en_bit=0;
  	const MODE_TTCVI = ASK_YES;
  	const MODE_TIM	 = ASK_NO;
 	
 	const int FV_MATCH_MAX = 10;
 	
 	/******/
 	char	filenm[50];
  	char	filenm0[50];
  	  	
  	int itime_now, fifo32;
  	
  	FILE *ofp;
  	/***/
 	
 		
 	

	printf("Scanning Trigger Rate and Monitoring for Veto\n");

	mode = askyn ("Use External (via NIM FP) rather than Internal Trigs? [Ext]", 'Y');

 	/* TIM Setup */
	reg[ENABLES3] |= bL1IDroECRen; /* en L1ID rollover to incr. ECR-ID */
	printf("Enabled L1ID rollover into ECR-ID.\n");
	
	reg[FV_TCOUNTL] = 0;
	reg[FV_COUNTL]  = 0;
	reg[FV_IDL]  	 = 0;

	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(4000000);
	reg[FV_P_MAX]   = convHz2Clk(15000);
	reg[FV_OFLOW]   = convHz2Clk(14999);
	reg[FV_DELTAT]  = 40;
	reg[FV_VETOLEN] = 3; /* 40000; *//* 1ms */
	
	reg[ENABLES] 	= 0;

	show_FFTV_status();

	
	if ( (mode == MODE_TTCVI)||(mode == ASK_DEFAULT) ) {
		show_ttcvi_trigrate_status(*ttc_CSR1, &freq_hz);
		trig_en_bit 	= bEnExtTRIG; 			/* Ext Trigs */
	}
	else {
		freq_hz = 80000;
		reg[CONTROL] = 0x00E0; /* 80 kHz */
		reg[ENABLES3] |= bRandom2En;
		trig_en_bit	 	= bEnIntTRIG; 	/* Int Trigs */
	}
	
	(void) ask_int("\nLoops per run (approx 1us per loop)[16s]?",0x1000000, &max_loops);

	time ( &time_now );
  	printf("Run: Doing %d loops, Start timestamp %s", max_loops, ctime (&time_now) );
	printf("----------------------------------------------------------");
	
	for (i=FV_MATCH_MAX; i>1; i--) {
  	
  		reg[FV_MATCH] = i;
  		printf("\n\n----------------------\n");
  		printf("New loop  FV_MATCH %d.\n\n",reg[FV_MATCH]);
  		
		reg[DEBUG_CTL] 	= 0x1000; /* ensure FFTV is reset */
		reg[DEBUG_CTL] 	= 0x0000; /* and on */
		
  		reg[L1IDH] 			= 0;
  		reg[L1IDL] 			= 0;
  		reg[FV_TCOUNTL] 	= 0;
		reg[FV_COUNTL] 	= 0;
		reg[FV_IDL] 	= 0;
		
	   diff_time = 0;
  		old_diff_time = 0;

		time ( &start_time );
  		printf ( "FVMatch %2d  Starting NOW! Timestamp %s", i, ctime (&start_time) );
  		
  		/**********/
  		itime_now = time(0);
		sprintf(filenm, "tp_fifo_");
		sprintf(filenm0, "%d", itime_now);
		strcat(filenm, filenm0);
		strcat(filenm, "_");
		sprintf(filenm0, "%d", i);
		strcat(filenm, filenm0);
		strcat(filenm, ".txt");
	
		ofp = fopen(filenm,"w");
		
		if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;

 	
 		exit_loop = 0;     
 	
 		time ( &time_now );
 		printf  (     "Start timestamp: %s", ctime (&time_now) );
  		fprintf (ofp, "Start timestamp: %s", ctime (&time_now) );
	
		printf("Running test in 'silence', check output file for errors. Ctrl-C to exit loop.\n");
		
		/***/
	
		get_and_show_fv_info(i, &l1id,
									&fv_countx, &fv_counth, &fv_countl,
									&fvtcount, 0, &fvid ); 
		
		loop = 0;	   
		reg[TP_FIFOL] = 0; /* clear fifos */
		reg[ENABLES] |= trig_en_bit; 	/* Go! */
		/* reg[DEBUG_CTL] |= bXrandomEn; */ 	/*** XGo! */
	   
  	   while (loop < max_loops) {
			
			if ( (DEBUGLVL>0)||(exit_loop==0) ) {
				if (loop%100000==0) { 
					get_and_show_fv_info(i, &l1id,
												&fv_countx, &fv_counth, &fv_countl,
												&fvtcount, (int)diff_time, &fvid ); 
								
				}
			}

			fifo32 = reg32[TP_FIFOL/2];
 		
			if ((fifo32&0x40000000)!=0x40000000) {  /* test for empty */
		
				if ((fifo32&0x80000000)==0x80000000) { /* test for full */
					/* printf (    "!ERROR! FIFO FULL!\n"); */
					printf("!");
   				fprintf(ofp, "!ERROR! FIFO FULL!\n");
				}
							
				fprintf(ofp, "%08x\n", fifo32&0x3FFFFFFF);
			}

		}

		/* Treat the next Control-C in default way (not needed on NT) */
  		if (signal (SIGINT, SIG_DFL) == SIG_ERR)
  			printf ("Os signal error (trying to un-catch ^C)\n") ;
			
		   	   
	   reg[ENABLES] = 0;		/* Trigs off */
  		reg[DEBUG_CTL] &= ~bXrandomEn; 	/*** Stop! */
  		
  		time_now = time(0);
 		printf  (     "Stop timestamp: %s", ctime (&time_now) );
  		fprintf (ofp, "Stop timestamp: %s", ctime (&time_now) );
  		fclose(ofp);
  		
	   	   
  	 	diff_time = difftime(time_now, start_time);
	   	   
		get_and_show_fv_info(i, &l1id, 
									&fv_countx, &fv_counth, &fv_countl,
									&fvtcount, (int)diff_time, &fvid ); 
		
  		printf ( "FVMatch %2d  End Timestamp %s", i, ctime (&time_now) );
  		printf ( "FVMatch %2d  DiffTime %d\n", i, (int)diff_time );
  		
  		afv_counth[i] 	= fv_counth;
  		afv_countl[i] 	= fv_countl;
  		afv_countx[i] 	= fv_countx;
  		afvtcount[i] 	= fvtcount;
  		afvid[i] 		= fvid;
  		al1id[i] 		= l1id;
  		adiff_time[i] = diff_time;
  		
	}
	
	printf("\n\nResults of Scan\n");
	printf("-------------------------------------------------------------------------------\n");
	printf("Loops %d  Freq %dHz  DeltaT %d  VetoLen %d\n", 
				max_loops, freq_hz, reg[FV_DELTAT], reg[FV_VETOLEN]+1); 
	printf(" N  Time   xL1ID    FVTrigCount  TotTrigs    FVCount        EstRat\n");
	
	for (i=2; i<=FV_MATCH_MAX; i++) {
		
			
		rat =	( ((float)(afv_countx[i])*0x10000*0x10000) 
				 +((float)(afv_counth[i])*0x10000)
				 +((float)(afv_countl[i]))
				)/(((float)adiff_time[i])*40000000);
		
		
		printf("%2d  %3d   %08x   %08x    %08x   %04x%04x%04x   %f\n", 
			i, (int)adiff_time[i], 
				al1id[i], afvtcount[i],
				(afvtcount[i]+al1id[i]),
				afv_countx[i], afv_counth[i], afv_countl[i], rat);
	
	}
	
	printf("-------------------------------------------------------------------------------\n");
}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */


void test_f(void)
{
	long int i, loops;
	time_t time_start, time_now;
	
	printf("\nTesting IO speed with a write then read for ~10 seconds.\n\n");
 	
 	time ( &time_start);
	
 	i = 0;
 	loops = 0;
 	
 	while (i < 1) {
 	
	 	time ( &time_now );
	 	if (time_now > (time_start + 10))  i=1;
	 	
	 	reg[L1IDL]=(int)loops;
	 	reg[L1IDH]=0;
	 		 	
	 	
	 	if ((loops/250000*250000) == loops) printf("I: %ld, %ld, %s", loops, time_now, ctime (&time_now));
	 	
	 	loops++;
  		
  	}
  	
	printf ( "Start timestamp: %ld, %s", time_start, ctime (&time_start));
	printf ( "End   timestamp: %ld, %s", time_now,   ctime (&time_now)  );
	printf ( "Loops: %ld.\n", loops);

}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void send_ttcvi_random_trigs (void)
{
	int freq, freq_hz;
	
	/* TIM setup */ /* Not needed - we are just SENDING Trigs here */
	
	/* reg[ENABLES3]   |= 0x0000;  */
	/* reg[COMMAND]    |= 0x1000; *//* RunMode */
	/* reg[RUN_ENABLE] |= 0x07FF; /* en All TTC Sigs */
 
	/* reg[ENABLES3] |= bL1IDroECRen; *//* en L1ID rollover to incr. ECR-ID */
	
	
	freq=8;
	
	while (freq>7) {
		(void) ask_int("\nTTCvi Trigger Freq (0=1Hz,1=100Hz,2=1k,3=5k,4=10k,5=25k,6=50k,7=100kHz)[6]",
					6, &freq);
	}
	printf("\n");


	/* TTCvi Setup */
	*ttc_reset = 0 ; 		 /* no data TTCvi reset */
	
	(void) wait_state_timeout(100, 0, 0, 1); /* wait a bit */

	/* *ttc_trigword_lo =  0x0300; *//* Size=1, Ext=1, Sub-add(7:0) = 0 */
	/* *ttc_trigword_hi  = 0x0007; *//* TTCrx address */

	*ttc_IDrst = 0; 				/* reset ID counter, I hope*/
	
	*ttc_L1lo = 0;
	*ttc_L1hi = 0;

	*ttc_CSR1 = 0x0000 + (freq<<12) + 0x04D; 
									/*Event Count, Freq, Reset FIFO, Random, L1A + Orbit */
	
	
	show_ttcvi_trigrate_status(*ttc_CSR1, &freq_hz);		

	*ttc_CSR2  = 0xFE00 ; 	/* reset + repeat 0 */

	/* *ttc_ladd  = 0x8000; */ 	/* long-format broadcast to TTCrx internal */
	/* *ttc_long  = 0x03B3; */ 	/* en-Clock40, en-Parbus, en-L1A   */


   printf("Sending Random TTC L1A's\n");
   
 
	 
} 

/* ------------------------------------------------------------------------- */
/* ========================================================================= */

void ttc_ttype_test (void)
{
	int nloops, i, ii, ttc_config_val, freq_hz;
	
	int l1idlfix, start_l1id, start_l1idl, l1idl, l1idh, tt_count, tts_count, diff, diff2;
	
	/* TIM setup */
	
	reg[ENABLES3]   |= 0x0000; /* set TIM TType-subaddr = 7 */
	reg[COMMAND]    |= 0x1000; /* RunMode */
	reg[RUN_ENABLE] |= 0x07FF; /* en All TTC Sigs */
 
	reg[ENABLES3] |= bL1IDroECRen; /* en L1ID rollover to incr. ECR-ID */
	

	/* TTCvi Setup */
		
	*ttc_reset = 0 ; 		 /* no data TTCvi reset */
	
	(void) wait_state_timeout(100, 0, 0, 1); /* wait a bit */
	

	*ttc_trigword_lo =  0x0300; /* Size=1, Ext=1, Sub-add(7:0) = 0 */
	*ttc_trigword_hi  = 0x0007; /* TTCrx address */

	*ttc_IDrst = 0; /* reset ID counter, I hope*/
	
	*ttc_L1lo = 0;
	*ttc_L1hi = 0;
					
	ttc_config_val = 0x704D; /*Event Count, Freq, Reset FIFO, Random, L1A + Orbit */
	
	*ttc_CSR1  = 0x704C  ;  /* Set to VME Trigs i.e, STOP trigs*/
	*ttc_trig = 0; /* send trig to ensure tim l1id is sensible */
	

	show_ttcvi_trigrate_status(ttc_config_val, &freq_hz);
	
	
	*ttc_CSR2  = 0xFE00 ; /* reset + repeat 0 */

	*ttc_ladd  = 0x8000 ; /* long-format broadcast to TTCrx internal */
	*ttc_long  = 0x03B3 ; /* en-Clock40, en-Parbus, en-L1A   */


	(void) ask_int("\nLoops [0x1000000]?",0x1000000, &nloops);
								 

	for (ii=0; ii<1; ii++) { /* long loop -- not always needed */
								 
		start_l1id = ((reg[L1IDH]&0x00FF)<<16)+reg[L1IDL]; /*start value*/
		printf("Start L1ID %x\n", start_l1id );
		start_l1idl = start_l1id & 0xFFFF;
		
		reg[TT_COUNT] = 0;
		reg[TTS_COUNT] = 0;
		
		*ttc_CSR1  = ttc_config_val;  /* Go! */
				
			
		for (i=0; i<nloops; i++) {
			
			if (i==(nloops-3)) *ttc_CSR1 = 0; /* disable triggers - wind down at end */
			
			l1idh = reg[L1IDH]; 
			l1idl = reg[L1IDL]; 
			tt_count = reg[TT_COUNT];
			tts_count = reg[TTS_COUNT];
		
			if (start_l1idl>l1idl)
				l1idlfix = l1idl + 0x10000;
			else 
				l1idlfix = l1idl;
			
		
			diff  = l1idlfix - tt_count - start_l1idl;
			diff2 = tt_count - tts_count;
			
			printf("l1id %04x%04x, tt_count %6d, tts_count %6d, diffs %+d, %+d, loop %d\n", l1idh, l1idl, tt_count, tts_count, diff, diff2, i);
			
			(void) wait_state_timeout(600, 0, 0, 1); /* wait ~ 1 second */

		}
		printf("\nTotal triggers (including rollover into ECR-ID): %d.\n", ((reg[L1IDH]<<16)+reg[L1IDL])-start_l1id);
		
		/* printf("l1id %04x%04x, tt_count %6d, tts_count %6d, diffs %+d, %+d, iiloop %d\n", 
					l1idh, l1idl, tt_count, tts_count, diff, diff2, ii); */
	}
								 
	

	 
	 
} 

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_tp_fifo(void)
{
	int fifol, fifoh, empty_loops, vetolvl = 0, dmode;
	
	int  fifo32, mode, xtrig_en_reg, xtrig_en_bit, freq, freq_hz;
	
 	
  	const int MODE_D32 = ASK_YES;
  	const int MODE_D16 = ASK_NO;
  	
  	char	filenm[50];
  	char	filenm0[50];
  	
  	time_t time_now;
  	int itime_now=123456789;
  	
  	FILE *ofp;

	itime_now = time(0);
	sprintf(filenm, "tp_fifo_");
	sprintf(filenm0, "%d", itime_now);
	strcat(filenm, filenm0);
	strcat(filenm, ".txt");
	
	ofp = fopen(filenm,"w");

	
	dmode = askyn("Use D32 to read fifo [D32]", 'Y');
	
	
	/*
	printf("\nSending Internal triggers and reading trig-period FIFO.\n");
	*/	
	/*reg[FREQUENCY] = 0x0a; 		*/	/* 30kHz */
	/*reg[ENABLES]   = bEnIntTRIG; */	/* ONLY Int Trigs on */	
	/*reg[ENABLES]   |= bEnRandom; */	/* Randomize! */	
		
	/*printf("Triggers: Random, IntTrig FREQUENCY register = %02x.\n\n", (reg[FREQUENCY]&0xFF));*/
	

	mode = askyn("Use 'Ext' (using NIM FP IO) rather than 'Int' Trigs. [Ext]", 'Y');
	printf("Mode: %d\n", mode);

	if ((mode==ASK_DEFAULT) || (mode==ASK_YES)) {
		printf("\nReceiving External Triggers and reading trig-period FIFO.\n");
		show_ttcvi_trigrate_status(*ttc_CSR1, &freq_hz);
		xtrig_en_reg   = ENABLES;
		xtrig_en_bit 	= bEnExtTRIG; 			/* Ext Trigs */
	}
	else {
		
		(void) ask_int("\nAverage Freq [0xB]?",0xB, &freq);
		
		
		reg[CONTROL] = ((freq&0xF)<<4) ; /* Set Rand2 Freq */
		
		/*trig_en_bit	 	= (bEnIntTRIG | bEnRandom); */	/* Random Int Trigs */
		reg[ENABLES3] |= bRandom2En;
		xtrig_en_reg   = ENABLES;
		xtrig_en_bit   = bEnIntTRIG;
		
	}

		
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;

 	empty_loops = 0;
 	exit_loop = 0;     
 	
 	time ( &time_now );
 	printf  (     "Start timestamp: %s", ctime (&time_now) );
  	fprintf (ofp, "Start timestamp: %s", ctime (&time_now) );
	
	printf("Running test in 'silence', check output file for errors. Ctrl-C to exit loop.\n");

	reg[TP_FIFOL] = 0; /* clear fifos */
	reg[xtrig_en_reg]   = xtrig_en_bit;	/* GO! = Only Ext Trigs on */	

   
   while (exit_loop == 0) {
   	

		if (dmode = MODE_D32) {
			fifo32 = reg32[TP_FIFOL/2];
		}
		else {
			
			fifol = reg[TP_FIFOL]&0xFFFF;
   		fifoh = reg[TP_FIFOH]&0xFFFF; 
  	
 			fifo32 = (fifoh*0x10000)+(fifol);
 		}
 		

	
		if ((fifo32&0x40000000)!=0x40000000) {  /* test for empty */
		
			if ((fifo32&0x80000000)==0x80000000) { /* test for full */
				/* printf (    "!ERROR! FIFO FULL!\n"); */
				printf("!");
   			fprintf(ofp, "!ERROR! FIFO FULL!\n");
			}
							
				
			fprintf(ofp, "%08x\n", fifo32&0x3FFFFFFF);
			/* printf("%d %u\n", vetolvl, fifo32&0x1FFFFFFF); */
				
			/* printf("%08x\n", fifo32&0x3FFFFFFF); */
								
				
			/*printf("count: %08x, raw fifo32: %08x, emloops %#4u\n",
							fifo32&0x1FFFFFFF, fifo32, empty_loops);*/

			/* empty_loops = 0; */
			
		}
		
		/* else {
			empty_loops ++;
		} */

	}

	/* Treat the next Control-C in default way (not needed on NT) */
  	if (signal (SIGINT, SIG_DFL) == SIG_ERR)
  		printf ("Os signal error (trying to un-catch ^C)\n") ;
	
	time_now = time(0);
 	printf  (     "Stop timestamp: %s", ctime (&time_now) );
  	fprintf (ofp, "Stop timestamp: %s", ctime (&time_now) );

  	

}


/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void twin_trig_burst(void) {
	
	int twin_period_clks = 4;  /*10;*/  /* 250 ns */
	int repeat_period_clks = 100; /*1000;*//* ~40kHz*2 */
	int burst_len = 0x100000; /* 1M trigs. */
	int d16_ram_address;
	int l1id = 0;
	
	printf("\nSending 1M twin-trigs (10 clks apart, 1k clk cycle=40Khz, 80kHz avg trig rate).\n\n");
	
	
	/* OOOPs burst mode doesn't work on Sequencer trigs in current firmware:
      will use L1ID polling instead (for now) */
   
   		
	/* reset sequencer */
	*reg_sequenc = 0x2200; /* (reg 0x1A=Sequencer Control) */

 	
 	/* ram_ptr = TIM_Base_Address + 0x8000) */
	/* note addressing is per word eg. ram(0) = address 00,01, ram(1) = 02,03 etc */
	/* this suits us because source ram is d0-d7(even address) and sink is d8-d15 (odd) */
	/* see bottom of the registers description for details */
	
	for (	d16_ram_address = 0; 
			d16_ram_address < (repeat_period_clks+1);
			d16_ram_address++)
		ram_ptr[d16_ram_address] = 0x0000 ; /* clear RAM */
		
	
	/* insert triggers into RAM */
	ram_ptr[0] = 0x0001;	                 /* can't address zero at the mo. - bug! */
	ram_ptr[twin_period_clks] = 0x0001; /* so add 1 - bug*/
	
	/* set repeat point of sequencer */
	*reg_seq_end = (repeat_period_clks-1);
	
 
	
	/* clear other trigger sources, just in case */
	*reg_enables &= 0xFDFD;  
	*reg_EnRun   &= 0xFFFD;  
	

	/* clear l1id counters */		
	*reg_L1lo = 0;
	*reg_L1hi = 0;

	

	/* setup burst */
	
	reg[BURST] = 5;
	reg[BURST_HI] = 0x0;
	
	reg[COMMAND] |= bvBurstMode;
	reg[COMMAND] &= ~bvBurstGo;
	

	
	/* GO! */
	*reg_sequenc = 0x0C01;  	/* Sequencer: cyclic, triggers, GO! */	
	
	
	reg[COMMAND] |= bvBurstGo;
	
		

	while (reg[STATUS]&bBurstBusy==0) {
		l1id =  (*reg_L1lo + ((*reg_L1hi & 0xFF) << 16)) & 0xFFFFFF;
		printf("L1ID %u", l1id);
		
	}
	
		
			
			
}


/* ------------------------------------------------------------------------- */
/* ========================================================================= */

void send_slow_tim_trigs(void)
{
	
	time_t time_now, time_old;
  	
  	double diff_time;
  	
	exit_loop = 0;     
 	
   time ( &time_now );
 	time_old = time_now;

   while (exit_loop == 0) {
   	 	
		time ( &time_now );	
		
		diff_time = difftime(time_now, time_old);
		
   	if (diff_time>2) {
		
			reg[COMMAND] = 0;
			reg[COMMAND] = 2;
			reg[COMMAND] = 0;
			
			printf("Trigger! (period = %ds) [Ctrl-C to exit loop]\n",
				diff_time);
				
			time_old = time_now;
		}

   }

}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
		















/* ======================================================================== */
/* ======================================================================== */
void timShowStatus(void)
{
	//printf("Status Not Working\n");
	
	int l1id;
	int status;

	(void)check_clk();
	(void)check_TTCclk();

	l1id = timGetL1ID( *reg_L1lo, *reg_L1hi );
	status =(int)*reg_status;
	 /*
	//printf("TIM_OK: ");
	//if ((status & TIM_OK) == 0) printf("NOT OK!"); else printf("OK!");

	printf("Internal Clock: ");
	if ((status & SAclkON) == 0) printf("FAILED!") else printf("Detected");

	printf("/n");
	printf("External Clock: ");
	if ((enables & 0x0100) == 0) printf("Disabled") else printf("Enabled");



 //Stand-alone clock OFF, status %04X, delays %04X\n",
   //         status, *reg_delays) ;
    ///error = 1 ;
  //}
//printf("TTC Clock :");
	

*/
	
	printf("Status Reg:%04x,  RunEnables:%04x,  Output:%04x.\n",
			*reg_status,*reg_EnRun,*reg_output);   
	printf("L1ID:%06x,  BCID:%03x,  BCin:%03x,  TTID:%03x.\n", 
	 			l1id, *reg_BCid, *reg_BCin, *reg_TTid);

}


/* ======================================================================== */
/* ======================================================================== */
void show_FFTV_status() {

	printf("\n\n");
	printf("FFTV Settings:\n");
	printf(" N Matches (FV_MATCH)         %4d\n", reg[FV_MATCH]);
	printf(" Upper Freq Limit (FV_P_MIN)  %4d (%3.3f kHz)\n", reg[FV_P_MIN],   convClk2Hz (reg[FV_P_MIN])/1000);
	printf(" Lower Freq Limit (FV_P_MAX)  %4d (%3.3f kHz)\n", reg[FV_P_MAX],	convClk2Hz (reg[FV_P_MAX])/1000);
	printf(" Tolerance (FV_DELTAT)        %4d (%3.3f us)\n",  reg[FV_DELTAT],  convClk2Sec(reg[FV_DELTAT])*1000000);
	printf(" Veto Duration (FV_VETOLEN)   %4d (%3.3f us)\n",  reg[FV_VETOLEN], convClk2Sec(reg[FV_VETOLEN])*1000000);

}


/* ======================================================================== */
/* ======================================================================== */
void show_ttcvi_trigrate_status( int ttc_config_val, int *freq_hz) 
{
	printf("TTC L1A Rate: ");
	if ((ttc_config_val & 0x7)    == 0x0005) {
		
	
		if ((ttc_config_val & 0x7000) == 0x0000) *freq_hz = 		1; 
		if ((ttc_config_val & 0x7000) == 0x1000) *freq_hz = 	 100;
		if ((ttc_config_val & 0x7000) == 0x2000) *freq_hz = 	1000;
		if ((ttc_config_val & 0x7000) == 0x3000) *freq_hz = 	5000;
		if ((ttc_config_val & 0x7000) == 0x4000) *freq_hz =  10000;
		if ((ttc_config_val & 0x7000) == 0x5000) *freq_hz =  25000;
		if ((ttc_config_val & 0x7000) == 0x6000) *freq_hz =  50000;
		if ((ttc_config_val & 0x7000) == 0x7000) *freq_hz = 100000;
		
		printf("Random, average %d Hz.\n", *freq_hz);
	}
}


/* ======================================================================== */
/* ======================================================================== */
void get_and_show_fv_info(	const int fv_match,
									int *l1id,
									int *fv_countx, 	int *fv_counth, int *fv_countl, 
									int *fv_tcount, unsigned int t, int *fvid
									)
{
	float rat, ffvc, ffvcx, ffvch, ffvcl;
	

	*fv_countl 	= reg[FV_COUNTL];
	*fv_counth 	= reg[FV_COUNTH];
	*fv_countx 	= reg[FV_COUNTX];
	*l1id 		= reg[L1IDH]*0x10000 		+ reg[L1IDL];
	*fv_tcount  = reg[FV_TCOUNTH]*0x10000 	+ reg[FV_TCOUNTL];
 	*fvid		= reg[FV_IDH]*0x10000 		+ reg[FV_IDL];
	
	ffvcl = (float)(*fv_countl);
	ffvch = (float)(*fv_counth)*0x10000;
	ffvcx = (float)(*fv_countx)*0x10000*0x10000;
	
	ffvc = ffvcx+ffvch+ffvcl;
	
	if (t==0) rat=0;
	else rat = ffvc/(t*40000000);
	

	printf("FVM %2d  XL1ID %08x  FVTC %08x  FVC %04x%04x%04x  R %f  FVID %08x\n",
			  fv_match, *l1id,     *fv_tcount, *fv_countx, 
															*fv_counth, 
															 *fv_countl,   rat,    *fvid );
							
}
/* ======================================================================== */
/* ======================================================================== */
