/* ---------------------------------------------------------------------------
 * Console for easily doing easy TIM things.
 * Adapted from 'timlet' and 'timrun' programs written by J.B.Lane.
 *
 * Matthew Warren Oct 2004 etc.                                   timConsole.c
 * ---------------------------------------------------------------------------
 */


#include "ask.h"
#include "vme.h"
#include "timlet.h"
#include "signal.h"
#include <string.h> /* strcat strcpy */
#include <time.h>
#include <math.h>  /* pow */

static int reset_level = 0 ;
static int exit_loop = 0;

const static unsigned short RAM_mask[1] = {0xFFFF} ;
unsigned int buffer[TIM_RAM_SIZE] ;


void timConsole    (void) ;
void timShowStatus (void);
void test_1 (void);
void test_2 (void);
void test_3 (void);
void test_4 (void);
void test_5 (void);
void test_6 (void);
void test_7 (void);
void test_8 (void);
void test_9 (void);
void test_a (void);
void test_b (void);
void test_c (void);
void test_fftv (void);
void fftv_scan_test (void);
void test_f (void);
void send_ttcvi_random_trigs (void);
void ttc_ttype_test (void);
void test_tp_fifo (void);
void twin_trig_burst (void);
void send_slow_tim_trigs (void);
void Random2_freq_test (void);
void RodBusy_Stop_Triggers_test (void);
void Scan_Sink_for_Orbit_Value (void);


void show_FFTV_status (void);
void show_ttcvi_trigrate_status(int ttc_config_val, int *freq_hz);
void get_and_show_fv_info(	const int fv_match,
									int *l1id,
									int *fv_countx, int *fv_counth, int *fv_countl, 
									int *fv_tcount, unsigned int t, int *fvid
									);
void ttcrx_i2c_register_dump(void);

void print_menu(void);


/* ------------------------------------------------------------------------- */

int main (void)
{
	unsigned int vme_base_addr ;
  	int r ;
  	
	printf("\n=== timConsole ===== (not rocket-science) ======\n\n");
	
  	(void) tim_vme (0x10000) ;
  	

 	if (memory_ptr == NULL) {
 		printf ("Error opening TIM VME window. EXITING!\n") ;
	}
	else {
		
 		ram_ptr = memory_ptr + TIM_RAM_SIZE ; /* offset is 16-bit words */
 		
 		r = ask ("Enter TTCvi VME base address", VME_BASE_TTC, &vme_base_addr) ;
  		ttc_ptr = vme_get_window (0x0800, vme_base_addr, VME_MODE_A24) ;
  		
  		if (ttc_ptr == NULL) {
  			printf ("Error opening TTCvi VME window, EXITING!\n") ;
  		}
 		else {
 			timConsole () ;
 		}
 	}

  	vme_end () ;
  	return (0) ;
}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void signal_handler (int sig)
{
	exit_loop = 1;
   printf ("\nExiting loop ASAP\n") ;
}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void timConsole (void)
{
	int ncommand = 0;
	int quit = 0;
	int indexL1A; // for check_Sink
	
		
	timSetupPtr();

	ttcviSetupPtr();
	
	profile_tim();
	
	printf("================================================\n");
	print_menu();
	
	while (quit < 1)
	{
		
		printf("\n--------------------------------------------------------------------------------");
		(void) ask_int("timConsole: Enter Choice (<Enter> show status & menu): ", 0, &ncommand);
		printf("--------------------------------------------------------------------------------\n");
		
		
		
		
		if (ncommand == 0) print_menu();
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 1) timReset(Loud);
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 2) {
			printf("Enabling TTC Clock (+ disabling ExtClk).\n");
			*reg_enables = (int)*reg_enables & 0xFEFF;
			*reg_EnRun = (int)*reg_EnRun | 0x0001;
		}
		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 3) {
			printf("Enabling External Clock (+ disabling TTCClk).\n");
			*reg_EnRun = (int)*reg_EnRun & 0xFFFE;
			*reg_enables = (int)*reg_enables | 0x0100;
		}
		
		
		/* ------------------------------------------------------------------- */	
		if (ncommand == 4) {
			printf("Enabling Internal Clock (by disabling TTC/Ext Clocks).\n");
			*reg_EnRun = (int)*reg_EnRun & 0xFFFE;
			*reg_enables = (int)*reg_enables & 0xFEFF;
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 5) {
			printf("Setting RunMode.\n");
			*reg_command = (int)*reg_command | 0x1000; // setrunmode
		}

		/* ------------------------------------------------------------------- */
	
		if (ncommand == 6) test_6();		
		
		/* ------------------------------------------------------------------- */
		if (ncommand == 7) {
			printf("Init Sequencer, ready for Trigger.\n");
			
			*reg_seq_end = 0x3FFF; //end value
			*reg_sequenc = 0x2200; //reset
			*reg_sequenc = 0x8000; //wait for next trig
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 8) {
			printf("Scanning the SinkRAM (checkSink).\n");
			checkSink (TIM_RAM_SIZE, NONE, &indexL1A);
		}

		
		/* ------------------------------------------------------------------- */
		if (ncommand == 9) {
			printf("Byeeeeee!\n\n");
			quit = 1;
		}
		
/* ------------------------------------------------------------------- */
		if (ncommand == 0x0a) test_a();
		if (ncommand == 0x0b) test_b();	
		if (ncommand == 0x0c) test_c();
		if (ncommand == 0x0d) test_fftv();
		if (ncommand == 0x0e) fftv_scan_test();
		if (ncommand == 0x0f) test_f();
		if (ncommand == 0x10) send_ttcvi_random_trigs();
		if (ncommand == 0x11) test_tp_fifo();
		if (ncommand == 0x12) twin_trig_burst();
		if (ncommand == 0x13) ttc_ttype_test();				
		if (ncommand == 0x14) send_slow_tim_trigs();
		if (ncommand == 0x15) ttcrx_i2c_register_dump();
		if (ncommand == 0x16) Random2_freq_test();
		if (ncommand == 0x17) RodBusy_Stop_Triggers_test();
		if (ncommand == 0x18) Scan_Sink_for_Orbit_Value();
	
	}
}




/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_6(void) {
	
	int i;
	
	int doublewidth = 0;

	printf("Leaving all signals running from Seq in cyclic mode.\n");
			
	for (i = 0 ; i < 16 ; i++) ram_ptr[i] = 0x0000 ;

	ram_ptr[ 0] = 0x0001;
	ram_ptr[ 2] = 0x0002;
	ram_ptr[ 4] = 0x0004;
	ram_ptr[ 6] = 0x0008;
	ram_ptr[ 8] = 0x0010;
	ram_ptr[10] = 0x0020;
	ram_ptr[12] = 0x0040;
	ram_ptr[14] = 0x0080;
	
	if (doublewidth == 1) {
		ram_ptr[ 1] = 0x0001;  /*0x0081;*/
		ram_ptr[ 3] = 0x0002;  /*0x0003;*/
		ram_ptr[ 5] = 0x0004;  /*0x0006;*/
		ram_ptr[ 7] = 0x0008;  /*0x000C;*/
		ram_ptr[ 9] = 0x0010;  /*0x0018;*/
		ram_ptr[11] = 0x0020;  /*0x0030;*/
		ram_ptr[13] = 0x0040;  /*0x0060;*/
		ram_ptr[15] = 0x0080;  /*0x00C0;*/
	}
	
	*reg_seq_end = 15;
	*reg_sequenc = 0x2200; //reset
	*reg_sequenc = 0x0CFF; //en-cyclic, go, all-sigs!
			
}


/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_a(void) {
	
	
	printf("Switching between TTC and Stand-alone clock (currently ");
	
	if (reg[ENABLES] & bEnExtClk) printf("External"); 
	else								  printf("Internal");
	
	printf(") via PLL-stable bit\n");
	printf("Exit loop with Control-C\n") ;
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;
		      
	exit_loop = 0;     
	while (exit_loop == 0) {
	   
	   timSwitchClk(TTC);
		timSwitchClk(Standalone);
	 }
	
	/* Treat the next Control-C in default way (not needed on NT) */
	 if (signal (SIGINT, SIG_DFL) == SIG_ERR)
	     printf ("Os signal error (trying to un-catch ^C)\n") ;
	

}

/* ------------------------------------------------------------------------- */
/* ========================================================================= */
void test_b(void) {
	
	unsigned long u1, u2 ;

	
	printf("Testing for FFTV by switch from Random to Fixed-Freq Trigs.\n");
	
	printf("Exit loop with Control-C\n") ;
	
	reg[FV_MATCH]  = 10;
	reg[FV_P_MIN]  = convHz2Clk(600000);
	reg[FV_P_MAX]  = convHz2Clk(110000);
	reg[FV_OFLOW]  = convHz2Clk(100000);
	reg[FV_DELTAT] = 2;
	
	show_FFTV_status();	
	
	u1 = /* reg[FV_COUNTX]*0x100000000 + */
			reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL];
			
	u2 = /*reg[BCOUNTX]*0x100000000 + */
	
			reg[BCOUNTH]*0x10000 + reg[BCOUNTL];
	
	printf(" FFTV Count (clks)(FV_COUNT*) %I64u\n", u1);
	printf(" Busy Count (clks)(BCOUNT*)   %I64u\n", u2);
		
		
	reg[DEBUG_CTL] |= bFVdisable;	 /* FFTV Off */				
	reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	
	
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR) {
		printf ("OS signal error (trying to catch ^C)\n") ;
	}
		      
	exit_loop = 0;     
	while (exit_loop == 0) {
	
		reg[ENABLES] &= ~bEnRandom;	/* Randomiser off */
		reg[FREQUENCY] = 0x04; 			/* 150kHz */
	
		(void) wait_state_timeout(50, 0, 0, 1); /* just a wait 0,0 will never be 1) */
	
		reg[ENABLES] |= bEnRandom; 	/* Randomiser on */
		reg[FREQUENCY] = 0x00; 			/* 600kHz - Randomiser reduces average rate by factor of 4 */
										
		(void) wait_state_timeout(50, 0, 0, 1);
	}
	
	/* Treat the next Control-C in default way (not needed on NT) */
	if (signal (SIGINT, SIG_DFL) == SIG_ERR) {
		printf ("Os signal error (trying to un-catch ^C)\n") ;
	}
	     
	show_FFTV_status();			
	
	u1 = /* reg[FV_COUNTX]*0x100000000 +*/ 
			reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL];
	printf(" FFTV Count (clks)(FV_COUNT*) %lu\n", u1);
	
	u1 = reg[BCOUNTX]*0xFFFFFFFF + reg[BCOUNTH]*0xFFFF + reg[BCOUNTL];
	printf(" Busy Count (clks)(BCOUNT*)   %lu\n", u1);

	
}



/* -------------------------------------------------------------------------*\
   Test C   																						  
   Test C  																						  
   Test C  																						  
\* =========================================================================*/
void test_c(void) {
	
	
	printf("Testing for FFTV by switch FFTV on and off.\n");
	
	printf("Exit loop with Control-C\n") ;
	
	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(600000);
	reg[FV_P_MAX]   = convHz2Clk(110000);
	reg[FV_OFLOW]   = convHz2Clk(100000);
	reg[FV_DELTAT]  = 2;
	reg[FV_VETOLEN] = 0x7FFF;
	
	show_FFTV_status();
				
	reg[ENABLES] |= bEnExtTRIG; 	/* Ext Trigs on */	
	
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;
		      
	exit_loop = 0;     
	while (exit_loop == 0) {

		reg[DEBUG_CTL] |= bFVdisable;	 /* FFTV Off */
		(void) wait_state_timeout(50, 0, 0, 1); /* just a wait 0,0 will never be 1) */
	
		reg[DEBUG_CTL] &= ~bFVdisable;	/* FFTV On */
		(void) wait_state_timeout(50, 0, 0, 1);
	}
	
	/* Treat the next Control-C in default way (not needed on NT) */
	if (signal (SIGINT, SIG_DFL) == SIG_ERR)
		printf ("Os signal error (trying to un-catch ^C)\n") ;

}


/* -------------------------------------------------------------------------*\
   Test D  																						  
   Test D  																						  
   Test D  																						  
\* =========================================================================*/

void test_fftv(void) {
	
	int quit = 0;
	int i2;
	unsigned long u1, u2 ;
  	time_t time_start, time_now;
  	double dift;

	printf("Monitoring for Veto'd Trigs when running Randoms.\n");
	printf("Exit loop with Control-C\n") ;
		
	reg[FV_TCOUNTL] = 0;
	reg[FV_COUNTL] = 0;

	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = convHz2Clk(40000000);
	reg[FV_P_MAX]   = convHz2Clk(15000);
	reg[FV_OFLOW]   = convHz2Clk(13000);
	reg[FV_DELTAT]  = 20;
	reg[FV_VETOLEN] = 40000; /* 1ms */
	
	show_FFTV_status();

	/* ---------------------------------------------------- */

	printf("\nRunning a few non-random trigs to prove working (osc=60kHz)\n");
	
	reg[DEBUG_CTL] = 0x1000; 			/* ensure FFTV is reset */
	reg[DEBUG_CTL] = 0x0000; 			/* and on */


	reg[ENABLES] &= ~bEnRandom; 		/* Randomiser off */
	reg[FREQUENCY] = 0x01; 				/* 60kHz */

	time ( &time_start );

	reg[ENABLES] |= bEnIntTRIG; 		/* Int Trigs on */	

		(void) wait_state_timeout(1000, 0, 0, 1); /* (~1000ms) delay of 1s */

	u1 = reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*+ reg[FV_COUNTX]*0x100000000*/
	
	time ( &time_now );
	dift = difftime(time_now, time_start);
	
	u2 = ((reg[L1IDH]&0x00FF)*0x10000) + reg[L1IDL];
	printf("Secs %04x, FVTCountL %04x, L1ID %7lu, FVCount %7lu\n", dift, reg[FV_TCOUNTL], u2, u1);

	/* ---------------------------------------------------- */

	printf("\nSwitching to random trigs, reg's reset (osc=300kHz)\n\n");
	
	reg[ENABLES] &= ~bEnIntTRIG; 	/* Int Trigs off */				
	
	reg[FV_TCOUNTL] = 0;
	reg[FV_COUNTL] = 0;
	reg[L1IDL] = 0;
	
	reg[FREQUENCY] = 0x02; 			/* 75kHz*4 = 300kHz */
	reg[ENABLES] |= bEnRandom; 	/* Randomiser on */

	reg[DEBUG_CTL] = 0x1000; /* ensure FFTV is reset */
	reg[DEBUG_CTL] = 0x0000; /* and on */

	time ( &time_start );
  	printf ( "Start timestamp: %s", ctime (&time_start) );

	reg[ENABLES] |= bEnIntTRIG; 	/* Int Trigs on */	

	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
   	printf ("OS signal error (trying to catch ^C)\n") ;
   
  	exit_loop = 0;     
   while (exit_loop == 0) {
 
		u1 = reg[FV_COUNTH]*0x10000 + reg[FV_COUNTL]; /*reg[FV_COUNTX]*0x100000000 + */
				
		if (u1 != u2) {
			time(&time_now);
			dift = difftime(time_now, time_start);
			u2 = ((reg[L1IDH]&0x00FF)*0x10000) + reg[L1IDL];
			i2 = reg[FV_TCOUNTL];
			printf("Secs %d, FVTCountL %d, L1ID %7lu, FVCount %7lu\n", dift, i2, u2, u1);

		}
		u2 = u1;
   }

	/* Treat the next Control-C in default way (not needed on NT) */
   if (signal (SIGINT, SIG_DFL) == SIG_ERR)
   	printf ("Os signal error (trying to un-catch ^C)\n") ;
   	
	time ( &time_now );
  	printf ( "End timestamp: %s", ctime (&time_now) );
}




/* -------------------------------------------------------------------------*\
   Test E     E     E     E     E     E     E     E     E     E     E     E
   Test E     E     E     E     E     E     E     E     E     E     E     E
   Test E     E     E     E     E     E     E     E     E     E     E     E
\* =========================================================================*/

void fftv_scan_test(void)
{
	const int DEBUGLVL = 0;
	
 	const int FV_MATCH_MAX = 9; 
	
	int i=0, mode=0, loop=0, max_loops=0;
	int l1id=0;
	int fv_counth=0,	 fv_countl=0,  fv_countx=0;
	int fvtcount=0;
	int fvid=0;
	
	int fullcount;
		 		 
	int freq_hz=0;
		
	double diff_time, old_diff_time;
	double trat, vrat;
	
	int 		al1id[16], 
				afv_counth[16],
				afv_countl[16], 
				afv_countx[16], 
				afvtcount[16],
				afvid[16],
				afullcount[16];
				
				
	double 	adiff_time[16];

	time_t time_now, start_time;
  	
  	int trig_en_bit=0, xtrig_en_bit=0;
  	const MODE_TTCVI = ASK_YES;
  	const MODE_TIM	 = ASK_NO;
 	
	int testmode = 0; /* 0 = no fifo, lot on screen; 1=fifo read */
 	
 	/******/
 	char	filenm[50];
  	char	filenm0[50];
  	char	filenmroot[50];
  	  	
  	int itime_now, fifo32;
  	
  	FILE *ofp;
  	/***/

 	/* --------------------------- end defs ---------------------- */
 	
	itime_now = time(0);
	sprintf(filenmroot, "tp_fifo_");
	sprintf(filenm0, "%d", (itime_now/60));
	strcat(filenmroot, filenm0);
	strcat(filenmroot, "_");
	
	printf("Root filename: %s\n", filenmroot);


	printf("Scanning Trigger Rate and Monitoring for Veto\n");

	mode = askyn ("Use External (via NIM FP) rather than Internal Trigs? [Ext]", 'Y');

 	/* TIM Setup */
	reg[ENABLES3] |= bL1IDroECRen; /* en L1ID rollover to incr. ECR-ID */
	printf("Enabled L1ID rollover into ECR-ID.\n");
	
	reg[FV_TCOUNTL] = 0;
	reg[FV_COUNTL]  = 0;
	reg[FV_IDL]  	 = 0;

	reg[FV_MATCH]   = 10;
	reg[FV_P_MIN]   = 80; /*500kHz*/  /*convHz2Clk(4000000);*/
	reg[FV_P_MAX]   = convHz2Clk(15000);
	reg[FV_OFLOW]   = convHz2Clk(13000);
	reg[FV_DELTAT]  = 40;
	reg[FV_VETOLEN] = 40000; /* 40000; *//* 1ms */
	
	reg[ENABLES] 	= 0;

	show_FFTV_status();

	
	if ( (mode == MODE_TTCVI)||(mode == ASK_DEFAULT) ) {
		show_ttcvi_trigrate_status(*ttc_CSR1, &freq_hz);
		trig_en_bit 	= bEnExtTRIG; 			/* Ext Trigs */
	}
	else {
		freq_hz = 80000;
		reg[CONTROL] = 0x00E0; /* 80 kHz */
		reg[ENABLES3] |= bRandom2En;
		trig_en_bit	 	= bEnIntTRIG; 	/* Int Trigs */
	}
	
	(void) ask_int("\nLoops per run (approx 2us per loop)[16s]?",0x300000, &max_loops);

	time ( &time_now );
  	printf("Run: Doing %d loops, Start timestamp %s", max_loops, ctime (&time_now) );
	printf("----------------------------------------------------------");
	
	/* for (i=2; i<=FV_MATCH_MAX; i++) { */
	for (i=2; i<=8; i++) {
  	
  		reg[FV_MATCH] = i;
  		printf("\n\n----------------------\n");
  		printf("New loop  FV_MATCH %d.\n\n",reg[FV_MATCH]);
  		
		reg[DEBUG_CTL] 	= 0x1000; /* ensure FFTV is reset */
		reg[DEBUG_CTL] 	= 0x0000; /* and on */
		
  		reg[L1IDH] 			= 0;
  		reg[L1IDL] 			= 0;
  		reg[FV_TCOUNTL] 	= 0;
		reg[FV_COUNTL] 	= 0;
		reg[FV_IDL] 	= 0;
		
	   diff_time = 0;
  		old_diff_time = 0;


		if (testmode ==1)	{
			strcpy(filenm, filenmroot);
			sprintf(filenm0, "%0d", i);
			strcat(filenm, filenm0);
			strcat(filenm, ".txt");
			printf("Filename: %s\n", filenm);
		
			ofp = fopen(filenm,"w");
			if (ofp==NULL) {
				printf("File open for write ERROR! Bye!\n");
			}
		}
		
		time ( &start_time );
  		printf ( "FVMatch %2d  Starting NOW! Timestamp %s", i, ctime (&start_time) );

		
		if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;

 	
 		exit_loop = 0;     
 	
	

		get_and_show_fv_info(i, &l1id,
									&fv_countx, &fv_counth, &fv_countl,
									&fvtcount, 0, &fvid ); 
									
		printf("\n");

		if (testmode==1)
		  	fprintf (ofp, "FVMatch %2d  Starting NOW! Timestamp %s", i, ctime (&start_time) );
			printf("Running test in 'silence', check output file for errors. Ctrl-C to exit loop.\n");
	
		/***/
	
 		time ( &start_time);

		fullcount=0;
		loop = 0;	   
		reg[TP_FIFOL] = 0; /* clear fifos */
		reg[ENABLES] |= trig_en_bit; 	/* Go! */
		/* reg[DEBUG_CTL] |= bXrandomEn; */ 	/*** XGo! */
	   
  	   while ( (fvid<(8000/i))&&(exit_loop==0) ) {
  	   	
  	 		time ( &time_now );
	  	 	diff_time = difftime(time_now, start_time);
			
			if (testmode == 0) {
				if (loop%100000==0) { 
					get_and_show_fv_info(i, &l1id,
												&fv_countx, &fv_counth, &fv_countl,
												&fvtcount, (int)diff_time, &fvid ); 
				}
			
			loop = (int)diff_time*200000-1; /* cludge to make times equal */
			
			}
			else {

				fifo32 = reg32[TP_FIFOL/2];
	 		
				if ((fifo32&0x40000000)!=0x40000000) {  /* test for empty */
					if ((fifo32&0x80000000)==0x80000000) { /* test for full */
						/*printf("!");*/
					
						/*printf (    "!ERROR! FIFO FULL!\n");*/
						/*fprintf(ofp, "!ERROR! FIFO FULL!\n");*/
						fullcount++;
					}
					
					/*printf (     "%08x\n", fifo32);*/
					fprintf(ofp, "%08x\n", fifo32);
				}
			}

			loop++;
		}
		if (testmode == 0) printf("\n");
		

		/* Treat the next Control-C in default way (not needed on NT) */
  		if (signal (SIGINT, SIG_DFL) == SIG_ERR)
  			printf ("Os signal error (trying to un-catch ^C)\n") ;
			
		   	   
	   reg[ENABLES] = 0;		/* Trigs off */
  		reg[DEBUG_CTL] &= ~bXrandomEn; 	/*** Stop! */
  		
  		time_now = time(0);
 		
		if (testmode ==1)	{
	  		fprintf (ofp, "!End! FIFO full count %d. Timestamp: %s", fullcount, ctime (&time_now) );
  			fclose(ofp);
  		}
  		
	   	   
  	 	diff_time = difftime(time_now, start_time);
	   	   
		get_and_show_fv_info(i, &l1id, 
									&fv_countx, &fv_counth, &fv_countl,
									&fvtcount, (int)diff_time, &fvid ); 
		printf("\n");
		
  		printf ("%2d,  Difftime %d, End Timestamp %s", i, (int)diff_time, ctime (&time_now) );
  		
  		afv_counth[i] 	= fv_counth;
  		afv_countl[i] 	= fv_countl;
  		afv_countx[i] 	= fv_countx;
  		afvtcount[i] 	= fvtcount;
  		afvid[i] 		= fvid;
  		al1id[i] 		= l1id;
  		adiff_time[i]  = diff_time;
  		afullcount[i] 	= fullcount;
  		
	}
	
	
	strcat(filenmroot, "results.txt");

	ofp = fopen(filenmroot,"w");

	
	printf("\n\nResults of Scan\n");
	fprintf(ofp,"\n\nResults of Scan\n");
	printf("-------------------------------------------------------------------------------\n");
	printf("Loops %d  Freq %dHz  DeltaT %d  VetoLen %d\n", 
				max_loops, freq_hz, reg[FV_DELTAT], reg[FV_VETOLEN]+1);   

	fprintf(ofp,"Loops %d  Freq %dHz  DeltaT %d  VetoLen %d\n", 
				max_loops, freq_hz, reg[FV_DELTAT], reg[FV_VETOLEN]+1);   


	printf (    " N Time xL1ID   FVTCount FVID      FVCount      Trat       Vrat    FUllcount\n");
	fprintf(ofp," N Time xL1ID   FVTCount FVID      FVCount      Trat       Vrat    FUllcount\n");
	
	for (i=2; i<=FV_MATCH_MAX; i++) {
		
			
		trat =( ((double)(afv_countx[i])*0x10000*0x10000) 
				 +((double)(afv_counth[i])*0x10000)
				 +((double)(afv_countl[i]))
				)/(((double)adiff_time[i])*40000000);
		
		
		vrat = (double)afvid[i]/(double)(afvtcount[i]+al1id[i]); 
		
		
		printf(     "%2d %3d %08x %08x %08x %04x%04x%04x %1.2e %1.2e  %d\n", 
					i, (int)adiff_time[i], 
					al1id[i], afvtcount[i],
					afvid[i],
					afv_countx[i], afv_counth[i], afv_countl[i], 
					trat, vrat, afullcount[i] 
				);
	
		fprintf(ofp,"%2d %3d %08x %08x %08x %04x%04x%04x %1.2e %1.2e  %d\n", 
					i, (int)adiff_time[i], 
					al1id[i], afvtcount[i],
					afvid[i],
					afv_countx[i], afv_counth[i], afv_countl[i], 
					trat, vrat , afullcount[i]
				);
				


	
	}
	fclose(ofp);

	
	printf("-------------------------------------------------------------------------------\n");
}

/* ------------------------------------------------------------------------- */

/* F         F          F          F          F         F          F       F */

/* ========================================================================= */


void test_f(void)
{
	long int i, loops;
	time_t time_start, time_now;
	
	printf("\nTesting IO speed with a write then read for ~10 seconds.\n\n");
 	
 	time ( &time_start );
	
 	i = 0;
 	loops = 0;
 	
 	while (i < 1) {
 	
	 	time ( &time_now );
	 	if (time_now > (time_start + 10))  i=1;
	 	
	 	reg[L1IDL]=(int)loops;
	 	reg[L1IDH]=0;
	 		 	
	 	
	 	if ((loops/250000*250000) == loops) printf("I: %ld, %ld, %s", loops, time_now, ctime (&time_now));
	 	
	 	loops++;
  		
  	}
  	
	printf ( "Start timestamp: %ld, %s", time_start, ctime (&time_start));
	printf ( "End   timestamp: %ld, %s", time_now,   ctime (&time_now)  );
	printf ( "Loops: %ld.\n", loops);

}




/* ------------------------------------------------------------------------- */



/* ========================================================================= */
void send_ttcvi_random_trigs (void)
{
	int freq, freq_hz;
	
	/* TIM setup */ /* Not needed - we are just SENDING Trigs here */
	
	/* reg[ENABLES3]   |= 0x0000;  */
	/* reg[COMMAND]    |= 0x1000; *//* RunMode */
	/* reg[RUN_ENABLE] |= 0x07FF; /* en All TTC Sigs */
 
	/* reg[ENABLES3] |= bL1IDroECRen; *//* en L1ID rollover to incr. ECR-ID */
	
	
	printf("Sending Random Triggers from TTCvi.\n\n");
	
	freq=8;
	
	while (freq>7) {
		(void) ask_int("TTCvi Trig Freq (0=1Hz,1=100,2=1k,3=5k,4=10k,5=25k,6=50k,7=100k)",
					6, &freq);
	}
	printf("\n");


	/* TTCvi Setup */
	*ttc_reset = 0 ; 		 /* no data TTCvi reset */
	
	(void) wait_state_timeout(100, 0, 0, 1); /* wait a bit */

	/* *ttc_trigword_lo =  0x0300; *//* Size=1, Ext=1, Sub-add(7:0) = 0 */
	/* *ttc_trigword_hi  = 0x0007; *//* TTCrx address */

	*ttc_IDrst = 0; 				/* reset ID counter, I hope*/
	
	*ttc_L1lo = 0;
	*ttc_L1hi = 0;

	*ttc_CSR1 = 0x0000 + (freq<<12) + 0x04D; 
									/*Event Count, Freq, Reset FIFO, Random, L1A + Orbit */
	
	
	show_ttcvi_trigrate_status(*ttc_CSR1, &freq_hz);		

	*ttc_CSR2  = 0xFE00 ; 	/* reset + repeat 0 */

	/* *ttc_ladd  = 0x8000; */ 	/* long-format broadcast to TTCrx internal */
	/* *ttc_long  = 0x03B3; */ 	/* en-Clock40, en-Parbus, en-L1A   */


   printf("Sending Random TTC L1A's\n");
   
 
	 
} 



/* ------------------------------------------------------------------------- */




/* ========================================================================= */

void ttc_ttype_test (void)
{
	int nloops, i, ii, ttc_config_val, freq_hz;
	
	int l1idlfix, start_l1id, start_l1idl, l1idl, l1idh, tt_count, tts_count, diff, diff2;
	
	/* TIM setup */
	
	reg[COMMAND]    |= 0x1000; /* RunMode */
	reg[RUN_ENABLE] |= 0x07FF; /* en All TTC Sigs */
 
	reg[ENABLES3] |= bL1IDroECRen; /* en L1ID rollover to incr. ECR-ID */
	

	/* TTCvi Setup */
		
	*ttc_reset = 0 ; 		 /* no data TTCvi reset */
	
	(void) wait_state_timeout(100, 0, 0, 1); /* wait a bit */
	

	*ttc_trigword_lo =  0x0300; /* Size=1, Ext=1, Sub-add(7:0) = 0 */
	printf("Using TTCrx=0 to broadcast TTypes.\n");
	*ttc_trigword_hi  = 0x0000; /* TTCrx=0 == broadcast */
	

	*ttc_IDrst 		= 0;		 /* reset ID counter, I hope*/
	
	*ttc_L1lo 		= 0;
	*ttc_L1hi 		= 0;
	
	*ttc_CSR1  		= 0x704C; /* Set to VME Trigs i.e, STOP trigs*/
	*ttc_trig 		= 0; 		 /* send trig to ensure tim l1id is sensible */
	
	ttc_config_val = 0x704D; /*Event Count, Freq, Reset FIFO, Random, L1A + Orbit */
	
	show_ttcvi_trigrate_status(ttc_config_val, &freq_hz);
	
	*ttc_CSR2  		= 0xFE00; /* reset + repeat 0 */

	*ttc_ladd  		= 0x8000; /* long-format broadcast to TTCrx internal */
	*ttc_long  		= 0x03B3; /* en-Clock40, en-Parbus, en-L1A   */


	(void) ask_int("\nLoops [0x1000000]?",0x1000000, &nloops);
								 

	for (ii=0; ii<1; ii++) { /* long loop -- not always needed */
								 
		start_l1id = ((reg[L1IDH]&0x00FF)<<16)+reg[L1IDL]; /*start value*/
		printf("Start L1ID %x\n", start_l1id );
		start_l1idl = start_l1id & 0xFFFF;
		
		reg[TT_COUNT] = 0;
		reg[TTS_COUNT] = 0;
		
		*ttc_CSR1  = ttc_config_val;  /* Go! */
				
			
		for (i=0; i<nloops; i++) {
			
			if (i==(nloops-3)) *ttc_CSR1 = 0; /* disable triggers - wind down at end */
			
			l1idh = reg[L1IDH]; 
			l1idl = reg[L1IDL]; 
			tt_count = reg[TT_COUNT];
			tts_count = reg[TTS_COUNT];
		
			if (start_l1idl>l1idl)
				l1idlfix = l1idl + 0x10000;
			else 
				l1idlfix = l1idl;
			
		
			diff  = l1idlfix - tt_count - start_l1idl;
			diff2 = tt_count - tts_count;
			
			printf("l1id %04x%04x, tt_count %6d, tts_count %6d, diffs %+d, %+d, loop %d\n", l1idh, l1idl, tt_count, tts_count, diff, diff2, i);
			
			(void) wait_state_timeout(600, 0, 0, 1); /* wait ~ 1 second */

		}
		printf("\nTotal triggers (including rollover into ECR-ID): %d.\n", ((reg[L1IDH]<<16)+reg[L1IDL])-start_l1id);
		
		/* printf("l1id %04x%04x, tt_count %6d, tts_count %6d, diffs %+d, %+d, iiloop %d\n", 
					l1idh, l1idl, tt_count, tts_count, diff, diff2, ii); */
	}
								 
	

	 
	 
} 

/* ------------------------------------------------------------------------- */




/* ========================================================================= */
void test_tp_fifo(void)
{
	int fifol, fifoh, empty_loops, vetolvl = 0, dmode;
	
	int  fifo32, mode, xtrig_en_reg, xtrig_en_bit, freq, freq_hz;
	
 	
  	const int MODE_D32 = ASK_YES;
  	const int MODE_D16 = ASK_NO;
  	
  	char	filenm[50];
  	char	filenm0[50];
  	
  	time_t time_now;
  	int itime_now=123456789;
  	
  	FILE *ofp;

	itime_now = time(0);
	sprintf(filenm, "tp_fifo_");
	sprintf(filenm0, "%d", itime_now);
	strcat(filenm, filenm0);
	strcat(filenm, ".txt");
	
	ofp = fopen(filenm,"w");

	dmode = askyn("Use D32 to read fifo [D32]", 'Y');
		
	/*
	printf("\nSending Internal triggers and reading trig-period FIFO.\n");
	*/	
	/*reg[FREQUENCY] = 0x0a; 		*/	/* 30kHz */
	/*reg[ENABLES]   = bEnIntTRIG; */	/* ONLY Int Trigs on */	
	/*reg[ENABLES]   |= bEnRandom; */	/* Randomize! */	
		
	/*printf("Triggers: Random, IntTrig FREQUENCY register = %02x.\n\n", (reg[FREQUENCY]&0xFF));*/
	

	mode = askyn("Use 'Ext' (using NIM FP IO) rather than 'Int' Trigs. [Ext]", 'Y');
	printf("Mode: %d\n", mode);

	if ((mode==ASK_DEFAULT) || (mode==ASK_YES)) {
		printf("\nReceiving External Triggers and reading trig-period FIFO.\n");
		show_ttcvi_trigrate_status(*ttc_CSR1, &freq_hz);
		xtrig_en_reg   = ENABLES;
		xtrig_en_bit 	= bEnExtTRIG; 			/* Ext Trigs */
	}
	else {
		
		(void) ask_int("\nAverage Freq [0xB]?",0xB, &freq);
		
		reg[CONTROL] = ((freq&0xF)<<4) ; /* Set Rand2 Freq */
		
		/*trig_en_bit	 	= (bEnIntTRIG | bEnRandom); */	/* Random Int Trigs */
		reg[ENABLES3] |= bRandom2En;
		xtrig_en_reg   = ENABLES;
		xtrig_en_bit   = bEnIntTRIG;
	}
	
	/* Trap the next ^C in signal_handler to interrupt the loop without an error */
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;

 	empty_loops = 0;
 	exit_loop = 0;     
 	
 	time ( &time_now );
 	printf  (     "Start timestamp: %s", ctime (&time_now) );
  	fprintf (ofp, "Start timestamp: %s", ctime (&time_now) );
	
	printf("Running test in 'silence', check output file for errors. Ctrl-C to exit loop.\n");

	reg[TP_FIFOL] = 0; /* clear fifos */
	reg[xtrig_en_reg]   = xtrig_en_bit;	/* GO! = Only Ext Trigs on */	

   
   while (exit_loop == 0) {

		if (dmode = MODE_D32) {
			fifo32 = reg32[TP_FIFOL/2];
		}
		else {
			
			fifol = reg[TP_FIFOL]&0xFFFF;
   		fifoh = reg[TP_FIFOH]&0xFFFF; 
  			fifo32 = (fifoh*0x10000)+(fifol);
 		}
 		
		if ((fifo32&0x40000000)!=0x40000000) {  /* test for empty */
		
			if ((fifo32&0x80000000)==0x80000000) { /* test for full */
				/* printf (    "!ERROR! FIFO FULL!\n"); */
				printf("!");
   			fprintf(ofp, "!ERROR! FIFO FULL!\n");
			}
							
			fprintf(ofp, "%08x\n", fifo32&0x3FFFFFFF);
			/* printf("%d %u\n", vetolvl, fifo32&0x1FFFFFFF); */
				
			/* printf("%08x\n", fifo32&0x3FFFFFFF); */
				
			/*printf("count: %08x, raw fifo32: %08x, emloops %#4u\n",
							fifo32&0x1FFFFFFF, fifo32, empty_loops);*/

			/* empty_loops = 0; */
		}
		/* else {
			empty_loops ++;
		} */
	}

	/* Treat the next Control-C in default way (not needed on NT) */
  	if (signal (SIGINT, SIG_DFL) == SIG_ERR)
  		printf ("Os signal error (trying to un-catch ^C)\n") ;
	
	time_now = time(0);
 	printf  (     "Stop timestamp: %s", ctime (&time_now) );
  	fprintf (ofp, "Stop timestamp: %s", ctime (&time_now) );

  	

}




/* ------------------------------------------------------------------------- */



/* ========================================================================= */
void twin_trig_burst(void) {
	
	int twin_period_clks = 4;  /*10;*/  /* 250 ns */
	int repeat_period_clks = 100; /*1000;*//* ~40kHz*2 */
	int burst_len = 0x100000; /* 1M trigs. */
	int d16_ram_address;
	int l1id = 0;
	
	printf("\nSending 1M twin-trigs (10 clks apart, 1k clk cycle=40Khz, 80kHz avg trig rate).\n\n");
	
	
	/* OOOPs burst mode doesn't work on Sequencer trigs in current firmware:
      will use L1ID polling instead (for now) */
   
   		
	/* reset sequencer */
	*reg_sequenc = 0x2200; /* (reg 0x1A=Sequencer Control) */

 	
 	/* ram_ptr = TIM_Base_Address + 0x8000) */
	/* note addressing is per word eg. ram(0) = address 00,01, ram(1) = 02,03 etc */
	/* this suits us because source ram is d0-d7(even address) and sink is d8-d15 (odd) */
	/* see bottom of the registers description for details */
	
	for (	d16_ram_address = 0; 
			d16_ram_address < (repeat_period_clks+1);
			d16_ram_address++)
		ram_ptr[d16_ram_address] = 0x0000 ; /* clear RAM */
		
	
	/* insert triggers into RAM */
	ram_ptr[0] = 0x0001;	                 /* can't address zero at the mo. - bug! */
	ram_ptr[twin_period_clks] = 0x0001; /* so add 1 - bug*/
	
	/* set repeat point of sequencer */
	*reg_seq_end = (repeat_period_clks-1);
	
	/* clear other trigger sources, just in case */
	*reg_enables &= 0xFDFD;  
	*reg_EnRun   &= 0xFFFD;  

	/* clear l1id counters */		
	*reg_L1lo = 0;
	*reg_L1hi = 0;

	/* setup burst */
	
	reg[BURST] = 5;
	reg[BURST_HI] = 0x0;
	
	reg[COMMAND] |= bvBurstMode;
	reg[COMMAND] &= ~bvBurstGo;
	
	/* GO! */
	*reg_sequenc = 0x0C01;  	/* Sequencer: cyclic, triggers, GO! */	

	reg[COMMAND] |= bvBurstGo;

	while (reg[BUSY_STAT3]&bBurstBusy3==0) {
		l1id =  (*reg_L1lo + ((*reg_L1hi & 0xFF) << 16)) & 0xFFFFFF;
		printf("L1ID %u", l1id);
		
	}
}
/* ------------------------------------------------------------------------- */



/* ========================================================================= */

void send_slow_tim_trigs(void)
{
	
	time_t time_now, time_old;
  	double diff_time;
  	
	/* --------------------------------------------------------- */
	exit_loop = 0;     
 	
   time ( &time_now );
 	time_old = time_now;

   while (exit_loop == 0) {
   	 	
		time ( &time_now );	
		diff_time = difftime(time_now, time_old);
		
   	if (diff_time>2) {
		
			reg[COMMAND] = 0;
			reg[COMMAND] = 2;
			reg[COMMAND] = 0;
			
			printf("Trigger! (period = %ds) [Ctrl-C to exit loop]\n",
				diff_time);
				
			time_old = time_now;
		}
   }
}

/* ------------------------------------------------------------------------- */




/* ========================================================================= */
		



/* ---------------------------------------------------------------------------
 */
void ttcrx_i2c_register_dump(void)
{

  int addr = 0 ;
  int data[29] ;

  int errors = 0;
  int i = 0;
  int abort_timeout = 0xFFFF;
  
  	printf("TTCrx register Dump (0:28):\n\n");
	/* 
	printf("Pre-abort TTCRX_CTL %04x\n",	reg[TTCRX_CTL]);
	
	reg[TTCRX_CTL] |= I2C_ABORT;
	
	while (i<abort_timeout) {
		printf("TTCRX_CTL %04x\n",	reg[TTCRX_CTL]);
	
		if ((reg[TTCRX_CTL] && I2C_ABORT) == 0) {
			printf("wait abort loops %d\n", i);
			i = abort_timeout;
		}
		i++;
	}
	*/
	
	printf("     0  1  2  3  4  5  6  7  8  9"); /*  a  b  c  d  e  f\n"); */
	
	for (addr = 0 ; addr <=28  ; addr ++ ) {
    
		errors = timTTCrxAccess (I2C_READ , addr, &data[addr]) ;
		
		if (((float)addr/10) == (float)(addr/10)) printf("\n %d: ", addr/10);
		if (errors != 0) printf ("ERROR!\n");
			
		printf("%02x ", data[addr]);
		/*printf("I2C addr %02d,  read data 0x%02x\n", addr, data) ;*/
	
	}

	printf("\n\nRegister Values:\n\n");
	printf(" Fine Delay 1             %02x\n", data[0]);
	printf(" Fine Delay 2             %02x\n", data[1]);
	printf(" Course Delay             %02x\n", data[2]);
	printf(" Control                  %02x\n", data[3]);
	printf(" Single Error Count     %02x%02x\n", data[9],data[8]);
	printf(" Double Error Count       %02x\n", data[10]);
	printf(" SEU Error Count <15:8>   %02x\n", data[11]);
	printf(" ID                     %02x%02x\n", (data[17] & 0x3F), data[16] );
	printf(" Master Mode A,B         %01x,%01x\n", ((data[17]>>6) & 0x3), ((data[18]>>6) & 0x3));
	printf(" I2C ID                   %02x\n", (data[18] & 0x3F));
	printf(" Configuration 1          %02x\n", data[19]);
	printf(" Configuration 2          %02x\n", data[20]);
	printf(" Configuration 3          %02x\n", data[21]);
	printf(" Status Register          %02x\n", data[22]);
	printf(" Bunch Counter          %02x%02x\n", data[25], data[24]);
	printf(" Event Counter        %02x%02x%02x\n", data[28], data[27], data[26]);
		  
}



/* -------------------------------------------------------------------------*\
   Test    16     16     16     16     16     16     16     16     16     16
   Test    16     16     16     16     16     16     16     16     16     16
   Test    16     16     16     16     16     16     16     16     16     16
\* =========================================================================*/

void Random2_freq_test(void)
{
	const int DEBUGLVL = 0;
 		
	int 				freqi=0;
	unsigned int 	l1id=0, l1id_max;
	int 				istarttime, idifftime, old_idifftime=0;
	double 			rate, realfreq;
	
	/*
	int 		al1id[16];
	double 	adiff_time[16];
  	*/

 	/* --------------------------- end defs ---------------------- */
	printf("Scanning Random2 Trigger Rates\n");
	
	reg[DEBUG_CTL] |= bFVdisable;	 /* FFTV Off */				
	reg[ENABLES3] |= bL1IDroECRen; /* en L1ID rollover to incr. ECR-ID */
	reg[ENABLES3] 	= 0;
	
	for (freqi=0; freqi<=15; freqi++) {
		
  		reg[L1IDH] 			= 0;
  		reg[L1IDL] 			= 0;

		if (freqi == 15) realfreq = 156250.00; /* 155 kHz*/
		if (freqi == 14) realfreq =  78125.00; /*  78 kHz*/
		if (freqi == 13) realfreq =  39062.50; /*  39 kHz*/
		if (freqi == 12) realfreq =  19531.25; /*  20 kHz*/
		if (freqi == 11) realfreq =   9765.63; /*  10 kHz*/
		if (freqi == 10) realfreq =   4882.81; /*   5 kHz*/
		if (freqi ==  9) realfreq =   2441.41; /*   2.5 kHz*/
		if (freqi ==  8) realfreq =   1220.70; /*   1.2 kHz*/
		if (freqi ==  7) realfreq =    610.35; /*   0.6 kHz*/
		if (freqi ==  6) realfreq =    305.18; /*   0.3 kHz*/
		if (freqi ==  5) realfreq =    152.59; /*   150 Hz*/
		if (freqi ==  4) realfreq =     76.29; /*    75 Hz*/
		if (freqi ==  3) realfreq = 	  38.15; /*    40 Hz*/
		if (freqi ==  2) realfreq = 	   9.54; /*    10 Hz*/
		if (freqi ==  1) realfreq = 	   4.77; /*     5 Hz*/
		if (freqi ==  0) realfreq = 	   1.19; /*     1.2 Hz*/
					
		reg[CONTROL] = (freqi << 4);	/* set frequency */
		
		l1id_max = (100*(int)(pow(2,freqi)));
	
		if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;

		istarttime=time(0);
		reg[ENABLES3] |= bRandom2En; 	/* Go! */
		
		/*printf("freqi %d(%03.4fkHz), l1id_max %d\n", freqi, realfreq, l1id_max);*/
	   
  	   while ( (l1id<l1id_max)&&(exit_loop==0) ) {
			
			l1id = reg[L1IDH]*0x10000 + reg[L1IDL];
			idifftime = (time(0)-istarttime);
			
			if (idifftime>old_idifftime) {
				
				if (idifftime>0) rate = ((double)l1id)/((double)idifftime)/1000;
				else rate = 0;
				
				printf("%2d: (%7.3f kHz), L1ID %8d of %8u, time %3d, rate %7.4f kHz\r", 
							freqi, realfreq, l1id, l1id_max, idifftime, rate );
							
				old_idifftime = idifftime;
				
			}
		}
	   
		/* Treat the next Control-C in default way (not needed on NT) */
  		if (signal (SIGINT, SIG_DFL) == SIG_ERR)
  			printf ("Os signal error (trying to un-catch ^C)\n") ;
			
		   	   
	   reg[ENABLES3] = 0;		/* Trigs off */
	   idifftime = (time(0)-istarttime);

		if (idifftime>0) rate = ((double)l1id)/((double)idifftime);
				else rate = 0;
				
		printf("%2d: (%7.3f kHz), L1ID %8d of %8u, time %3d, rate %7.3f kHz\n", 
					freqi, realfreq, l1id, l1id_max, idifftime, rate );
				
		old_idifftime = 0;
	}
}


/* -------------------------------------------------------------------------*\
   Test    17     17     17     17     17     17     17     17     17     17
     Test    17     17     17     17     17     17     17     17     17
       Test    17     17     17     17     17     17     17     17     17
\* =========================================================================*/

void RodBusy_Stop_Triggers_test(void) {
	
	unsigned int i, n;
	unsigned int l1id, bcount;
	unsigned int tnobusy=0, tbusy=0, tdiff=0,  theoretical=0;
	
	unsigned int seq_len = 16000;
	unsigned int loops = 100;

	/* ---------------------------------------------------------------------- */
	printf("Testing if RodBusy Stops Triggers.\n");
	printf("Sending %dx%d spares per loop to TimTestBoards (thus back as RodBusys).\n\n",seq_len, loops);
	
	reg[DEBUG_CTL] = 0x1010;       /***** 0x1200;*/ 			/* FFTV off, no SA mode in rod-busy */
	reg[FREQUENCY] = 0x00; 				/* 600kHz */
	
	reg[BUSY_EN3] = 0x0001;
	reg[ENABLES3] |= bL1IDroECRen; /* en L1ID rollover to incr. ECR-ID */
	
	for (	i=0; i < seq_len; i++)
		ram_ptr[i] = 0x0080 ; 		/* fill some ram with Spares */

	reg[SEQ_END] = seq_len-1;
	reg[SEQ_CTL] = 0x0200; 	/* seq reset */

	for (n=0; n<10; n++) {

		if ((n % 2) == 0) {
			printf("RodBusy's DISABLED\r");
			reg[RB_MASK] = 0x0000;
		}
		
		if ((n % 2) == 1) {
			printf("RodBusy's ENABLED\r");
			reg[RB_MASK] = 0xFFFF;
		}		
	
		reg[BCOUNTL] = 0; 				/* reset busy-counter */
		reg[L1IDL] 	 = 0x0;  			/* reset l1id to 0	 */
		reg[L1IDH] 	 = 0x0;  			/* reset liid to 0    */

		l1id 		= (reg[L1IDH]<<16) + reg[L1IDL];
		bcount	= (reg[BCOUNTH]<<16) + reg[BCOUNTL];
		
		/* printf("%d Strt:    L1ID %12d, BCOUNT %12d\n", n, l1id, bcount); */
	
		wait_secs(1); /* Start on sec clock! */
	
		/* Go ! */	
		reg[ENABLES] = bEnIntTRIG; 	/* Int Trigs on */
			
		for (i=1; i<=loops; i++) {
			reg[SEQ_CTL] = 0x00FF; 	/* enable all seq outputs */
			reg[SEQ_CTL] |= bSeqGo; /* send spares */
			wait_secs(1);	
	
			l1id 		= (reg[L1IDH]<<16) + reg[L1IDL];
			bcount	= (reg[BCOUNTH]<<16) + reg[BCOUNTL];
			
			printf("%3d of %3d  l1id %10d, bcount %10d\r", i, loops, l1id, bcount);
			reg[SEQ_CTL] = 0x0200; 	/* seq reset */
		}
		
		reg[ENABLES] = 0; 	/* Trigs off */	
		reg[SEQ_CTL] = 0x0200; 	/* seq reset */
			
		l1id 		= (reg[L1IDH]<<16) + reg[L1IDL];
		bcount	= (reg[BCOUNTH]<<16) + reg[BCOUNTL];
	
		printf("%d Stop:     l1id %10d, bcount %10d\n", 
					n, l1id, bcount);
					
		if ((n % 2) == 0) tnobusy += l1id;
		if ((n % 2) == 1) tbusy += l1id;
	}

	tdiff = tnobusy - tbusy;
	theoretical = (int)( (n/2)*loops*seq_len*0.000000025*600000);
	
 	printf("\nTotal Trigs: Rodbusy On %u, Off %u, Diff %u, theory %u\n", 
 				tnobusy, tbusy, tdiff,  theoretical );

}



/* ------------------------------------------------------------------------- *\
	18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18
	 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18
	  18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18
   18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18	   
\* ========================================================================= */
void Scan_Sink_for_Orbit_Value(void)
{
	int i, last_i, loop;
	int orbit, diff_orbit=0;
	int errors=0;
	int required_orbit = 3564;
	
	printf("\nUsing Sink looking for Orbit (inter BCR) Values.\n\n");
 		
 	 	
 	reg[SEQ_CTL] = bSeqReset | bSinkReset;
 	reg[SEQ_END] = 0xFFFF;
 	reg[RUN_ENABLE] = 0x9;
 	
	
	if (signal (SIGINT, signal_handler) == SIG_ERR)
		printf ("OS signal error (trying to catch ^C)\n") ;


 	while (exit_loop==0) {
 	
 	
 		reg[SEQ_CTL] = bSinkGo;
		(void) wait_state_timeout(50, STATUS, bSeqSinkAct, 0);
	

 		loop = 0;			
 		last_i = 0;
 		
 		for (i=0; i<0x4000; i++) {
 			
 			/*
 			if ((i%1000)==0)
 				printf("i %d, ram_ptr[i] %4x\n",i, ram_ptr[i]);
 			
 			*/
 		
			if ((ram_ptr[i] & 0x0400)==0x0400) {
				
				/*
				printf("Found One! i %d, ram_ptr[i] %4x\n",i, ram_ptr[i]);
				*/
 		
 				if (loop > 0) {
 		
 					orbit = (i-last_i);

					diff_orbit = (required_orbit-orbit);
 									
 					printf("BCR at %5d, Orbit %4d, Diff %2d, Errors %d\r", 
 								i, orbit, diff_orbit, errors );
 					 					
 					if (diff_orbit != 0) {
 						printf("\nERROR! Required Orbit %d, Scan Orbit %d, Diff %d\n",
 									required_orbit, orbit, diff_orbit);
 						errors++;
 					}
 				}
 				last_i = i;
 				loop++;
 			}
 			
 		}
	 	reg[SEQ_CTL] = bSeqReset | bSinkReset;
	}
	/* Treat the next Control-C in default way (not needed on NT) */
	 if (signal (SIGINT, SIG_DFL) == SIG_ERR)
		printf ("Os signal error (trying to un-catch ^C)\n") ;
}







/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */

void timShowStatus(void)

{
	//printf("Status Not Working\n");
	
	int l1id;
	int status;

	(void)check_clk();
	(void)check_TTCclk();

	l1id = timGetL1ID( *reg_L1lo, *reg_L1hi );
	status =(int)*reg_status;
	 /*
	//printf("TIM_OK: ");
	//if ((status & TIM_OK) == 0) printf("NOT OK!"); else printf("OK!");

	printf("Internal Clock: ");
	if ((status & SAclkON) == 0) printf("FAILED!") else printf("Detected");

	printf("/n");
	printf("External Clock: ");
	if ((enables & 0x0100) == 0) printf("Disabled") else printf("Enabled");



 //Stand-alone clock OFF, status %04X, delays %04X\n",
   //         status, *reg_delays) ;
    ///error = 1 ;
  //}
//printf("TTC Clock :");
	

*/
	
	printf("Status Reg:%04x,  RunEnables:%04x,  Output:%04x.\n",
			*reg_status,*reg_EnRun,*reg_output);   
	printf("L1ID:%06x,  BCID:%03x,  BCin:%03x,  TTID:%03x.\n", 
	 			l1id, *reg_BCid, *reg_BCin, *reg_TTid);

}


/* ======================================================================== */
/* ======================================================================== */
void show_FFTV_status() {

	printf("\n\n");
	printf("FFTV Settings:\n");
	printf(" N Matches (FV_MATCH)          %4d\n", reg[FV_MATCH]);
	printf(" Upper Freq Limit (FV_P_MIN)   %4d (%3.3f kHz)\n", reg[FV_P_MIN],   convClk2Hz (reg[FV_P_MIN])/1000);
	printf(" Lower Freq Limit (FV_P_MAX)   %4d (%3.3f kHz)\n", reg[FV_P_MAX],	convClk2Hz (reg[FV_P_MAX])/1000);
	printf(" Period Wrap Level (FV_OFLOW)  %4d (%3.3f kHz)\n", reg[FV_OFLOW],	convClk2Hz (reg[FV_OFLOW])/1000);
	printf(" Tolerance (FV_DELTAT)         %4d (%3.3f us)\n",  reg[FV_DELTAT],  convClk2Sec(reg[FV_DELTAT])*1000000);
	printf(" Veto Duration (FV_VETOLEN)    %4d (%3.3f us)\n",  reg[FV_VETOLEN], convClk2Sec(reg[FV_VETOLEN])*1000000);

}


/* ======================================================================== */
/* ======================================================================== */
void show_ttcvi_trigrate_status( int ttc_config_val, int *freq_hz) 
{
	printf("TTC L1A Rate: ");
	if ((ttc_config_val & 0x7)    == 0x0005) {
		
	
		if ((ttc_config_val & 0x7000) == 0x0000) *freq_hz = 		1; 
		if ((ttc_config_val & 0x7000) == 0x1000) *freq_hz = 	 100;
		if ((ttc_config_val & 0x7000) == 0x2000) *freq_hz = 	1000;
		if ((ttc_config_val & 0x7000) == 0x3000) *freq_hz = 	5000;
		if ((ttc_config_val & 0x7000) == 0x4000) *freq_hz =  10000;
		if ((ttc_config_val & 0x7000) == 0x5000) *freq_hz =  25000;
		if ((ttc_config_val & 0x7000) == 0x6000) *freq_hz =  50000;
		if ((ttc_config_val & 0x7000) == 0x7000) *freq_hz = 100000;
		
		printf("Random, average %d Hz.\n", *freq_hz);
	}
}


/* ======================================================================== */
/* ======================================================================== */
void get_and_show_fv_info(	const int fv_match,
									int *l1id,
									int *fv_countx, 	int *fv_counth, int *fv_countl, 
									int *fv_tcount, unsigned int t, int *fvid
								 )
{
	double trat, rat, ffvc, ffvcx, ffvch, ffvcl;
	

	*fv_countl 	= reg[FV_COUNTL];
	*fv_counth 	= reg[FV_COUNTH];
	*fv_countx 	= reg[FV_COUNTX];
	*l1id 		= reg[L1IDH]*0x10000 		+ reg[L1IDL];
	*fv_tcount  = reg[FV_TCOUNTH]*0x10000 	+ reg[FV_TCOUNTL];
 	*fvid		= reg[FV_IDH]*0x10000 		+ reg[FV_IDL];
	
	ffvcl = (double)(*fv_countl);
	ffvch = (double)(*fv_counth)*0x10000;
	ffvcx = (double)(*fv_countx)*0x10000*0x10000;
	
	ffvc = ffvcx+ffvch+ffvcl;
	
	if ((*l1id)==0) rat=0; 
	else rat = ((double)(*fvid))/((double)(*l1id)+(double)(*fv_tcount));
	
	
	if (t==0) trat=0; 
	else trat = ((double)ffvc)*0.000000025/(double)t;
		

	printf("N%d L1 %08x VTC%08x VC%04x%04x%04x VID%08x Rt%2.2e R%2.2e\r",
			  fv_match,   *l1id,
			       		  *fv_tcount, *fv_countx, 
												*fv_counth, 
												 *fv_countl, *fvid, trat, rat );
							
}
/* ======================================================================== */
/* ======================================================================== */

void print_menu()
{
	printf("\n-- Status --------------------------------------\n");
 	
 	timShowStatus();

	printf("\n-- Menu ----------------------------------------\n");
	printf("  0. Check Status\n");
	printf("  1. Reset TIM\n");	
	printf("  2. Enable TTC Clock (fibre)\n");	
	printf("  3. Enable External Clock (front panel lemo)\n");	
	printf("  4. Enable Internal Clock (disable Ext/TTC Clocks)\n");
	printf("  5. Set RunMode\n");		
	printf("  6. Run All signals from Sequencer (cyclic)\n");
	printf("  7. Reset Sink - Ready for Triggers\n");
	printf("  8. Dump Sink RAM\n");
	printf("  9. Quit\n");
	printf("\n");
	printf("  a. TTC to SA Repeditive Clock Switch Test (for scoping)\n");
	printf("  b. Run FFTV Random/Fixed Freq Test\n");
	printf("  c. Run FFTV Enable/Disable Test\n");

	printf("  d. Run Random Trigs, FFT Count Monitor Test\n");
	printf("  e. Run FFTV scan test\n");
	printf("  f. 10 Second IO Speed test\n");
	printf(" 10. Send TTCvi Random Trigs\n");
	printf(" 11. Test trigger Period FIFO\n");
	printf(" 12. Send Twin Trigs Burst\n");
	printf(" 13. TTC Type Test\n");
	printf(" 14. Send Slow TIM Trigs\n");
	printf(" 15. TTCrx I2C Register Dump\n");
	printf(" 16. Random2 Frequencies Check\n");
	printf(" 17. RodBusy Stopping Triggers Test\n");
	printf(" 18. Scan Seq for Orbit Values\n");

	printf("\n");
}


